;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       avg_dark_nb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
; 
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
; 
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION construct_initial_guess_new, phi, bb, theta_err, ical_ret, ical_pol, cpolang, cretang, $
                                  TPARAMS = tparams

     radeg   = !pi/180.

     phi2 = phi

     umtx = setmtx(   1.0000,   0.0000,   0.0000,    0.0000  $
                    , 0.0000,   1.0000,   0.0000,    0.0000  $
                    , 0.0000,   0.0000,   1.0000,    0.0000  $
                    , 0.0000,   0.0000,   0.0000,    1.0000  )

;;----------------------------------
;; calculate CU Mueller matrices
;;----------------------------------

;; the linear polarizer

     px = 1
     py = 0. ;;; 0.005
     a = ( px^2 + py^2 )
     b = ( px^2 - py^2 )
     d  = 2. * px * py 

     polmtx = setmtx(  a,   b,   0.,  0.  $
                     , b,   a,   0.,  0.  $
                     , 0.,  0.,  d,   0.  $
                     , 0.,  0.,  0.,  d   )

     polmtx = polmtx * 0.5 

     polmtx = transpose(polmtx)

;; the retarder

     px = 1 + bb/2.
     py = 1 - bb/2.

     sq = sqrt(px*py)
     a  = ( px + py )/2.
     c  = ( px - py )/2.

     sd = sin( phi2 * radeg )
     cd = cos( phi2 * radeg  )

     retmtx = setmtx(  a,    c,     0.,     0.   $
                     , c,    a,     0.,     0.   $
                     , 0.,  0.,  sq*cd,  -sq*sd  $
                     , 0.,  0.,  sq*sd,  sq*cd   )

     retmtx = transpose(retmtx)

;;-----------------------------------------------
;; do the loop over measurements
;;-----------------------------------------------

     ncal = n_elements(cretang)

     stksin = FLTARR(ncal,4)

     FOR ical = 0, ncal - 1 DO BEGIN

         if keyword_set(TPARAMS) then begin

            tmtx  = ibis_tmtx(tparams.values, (tparams.lambda)[0], $ 
                              (tparams.azi[ical])[0], (tparams.elev[ical])[0], (tparams.table[ical])[0] )

         endif else tmtx = umtx

         vecout = transpose(tmtx ## [1.,0,0,0])

         ;;----------------------------------
         ;; calculate CU Mueller matrices
         ;;----------------------------------

         ;;----------------------------------
         ;; linear polarizer is in the beam
         ;;----------------------------------

         if ical_pol[ical] eq 1 then begin

            ;; rotation of polarizer

            c = cos( 2. * cpolang[ical] * radeg )
            s = sin( 2. * cpolang[ical] * radeg )

            rot1 = setmtx(   1,   0.,   0.,  0.  $
                           , 0.,   c,   s,   0.  $
                           , 0.,  -s,   c,   0.  $
                           , 0.,   0.,  0.,  1   )

            rot1 = transpose(rot1)

            rot2 = setmtx(  1,   0.,   0.,  0.  $
                          , 0.,   c,  -s,   0.  $
                          , 0.,   s,   c,   0.  $
                          , 0.,   0.,  0.,  1   )
 
            rot2 = transpose(rot2)

            ;; rotation to fast axis

            vecout = rot1 ## vecout

            ;; apply polarizer

            vecout = polmtx ## vecout

            ;; rotate back 

            vecout = rot2 ## vecout

            ;;----------------------------------
            ;; retarder is in the beam
            ;;----------------------------------

            if ical_ret[ical] eq 1 then begin

               ;; rotation of retarder

               c = cos( 2. * (cretang[ical] + theta_err) * radeg )
               s = sin( 2. * (cretang[ical] + theta_err) * radeg )

               rot1 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,   s,   0.  $
                             , 0.,  -s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )

               rot1 = transpose(rot1)


               rot2 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,  -s,   0.  $
                             , 0.,   s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )
 
               rot2 = transpose(rot2)

               ;; rotation to fast axis

               vecout = rot1 ## vecout

               ;; apply retarder

               vecout = retmtx ## vecout

               ;; rotate to input frame

               vecout = rot2 ## vecout

            endif

         endif

         if ical_pol[ical] eq 0 then begin

            if ical_ret[ical] eq 1 then begin

               ;; rotation of retarder

               c = cos( 2. * (cretang[ical] + theta_err) * radeg )
               s = sin( 2. * (cretang[ical] + theta_err) * radeg )

               rot1 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,   s,   0.  $
                             , 0.,  -s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )

               rot1 = transpose(rot1)


               rot2 = setmtx(  1,   0.,   0.,  0.  $
                             , 0.,   c,  -s,   0.  $
                             , 0.,   s,   c,   0.  $
                             , 0.,   0.,  0.,  1   )
 
               rot2 = transpose(rot2)

               ;; rotation to fast axis

               vecout = rot1 ## vecout

               ;; apply retarder

               vecout = retmtx ## vecout

               ;; rotate to input frame

               vecout = rot2 ## vecout

            endif

            if ical_ret[ical] eq 0 then begin

               print,'There is something wrong, neither retarder nor polarizer are in the beam!'
               stop

            endif

         endif

      ;;---------------------------------
      ;; 6. normalize
      ;;---------------------------------

          stks             = vecout ; / vecout[0] 
          stksin[ ical, *] = REFORM( stks )

     ENDFOR

RETURN, stksin
END
