;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_calbrate_xt
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_calibrate_xt, stksin, xmatl, xmatr, tmat, stksout, $
                       DUAL = dual, SINGLE = single

;-------------------------------------------------
;; Invert T-Matrix
;-------------------------------------------------

     Tinv = INVERT(tmat) 
     Tinv = Tinv/Tinv[0,0]

;;-------------------------------------------------
;; Dual-beam spectrpolarimetry
;;-------------------------------------------------

     IF KEYWORD_SET(DUAL) THEN BEGIN

        ;;-------------------------------------------------
        ;; Create Arrays
        ;;-------------------------------------------------

        s = SIZE(stksin)

        stksinl = REFORM( stksin[*,*,*,*,0] )
        stksinr = REFORM( stksin[*,*,*,*,1] )

        stksoutl = stksinl
        stksoutr = stksinr
  
        stksout = fltarr( s[1], s[2], s[3], 4)

        ;;-------------------------------------------------
        ;; Invert X-matrices
        ;;-------------------------------------------------

        Xinv_right = INVERT(xmatr)
        Xinv_left  = INVERT(xmatl)

        ;;----------------------------------------------------
        ;; Loop over data ---> Do Calibration for X and T
        ;;----------------------------------------------------

        s = SIZE(stksinr)

        FOR i = 0, s[1] - 1 DO BEGIN

            FOR j = 0, s[2] - 1 DO BEGIN

                FOR l = 0, s[3]-1 DO BEGIN

                    ar = REFORM( stksinr[i,j,l,*] )
                    br = Xinv_right ## ar
                    cr = Tinv ## br
                    al = REFORM( stksinl[i,j,l,*] )
                    bl = Xinv_left ## al
                    cl = Tinv ## bl
                    stksoutl[i,j,l,*] = cl
                    stksoutr[i,j,l,*] = cr

                ENDFOR

            ENDFOR

        ENDFOR

        ;;-------------------------------------------------
        ;; Combine calibrated left and right beam
        ;;-------------------------------------------------

        stksout[*,*,*,0] = REFORM( 0.5 * ( stksoutl[*,*,*,0] + stksoutr[*,*,*,0] ) )
        stksout[*,*,*,1] = REFORM( 0.5 * ( stksoutl[*,*,*,1] + stksoutr[*,*,*,1] ) )
        stksout[*,*,*,2] = REFORM( 0.5 * ( stksoutl[*,*,*,2] + stksoutr[*,*,*,2] ) )
        stksout[*,*,*,3] = REFORM( 0.5 * ( stksoutl[*,*,*,3] + stksoutr[*,*,*,3] ) )

     ENDIF

;;-------------------------------------------------
;; Single-beam spectropolarimetry
;;-------------------------------------------------

     IF KEYWORD_SET(SINGLE) THEN BEGIN

        ;;-------------------------------------------------
        ;; Create Arrays
        ;;-------------------------------------------------

        s = SIZE(stksin)

        stksout = fltarr( s[1], s[2], s[3], 4)

        ;;-------------------------------------------------
        ;; Invert X-matriX
        ;;-------------------------------------------------

        Xinv = INVERT(xmatl)

        ;;----------------------------------------------------
        ;; Loop over data ---> Do Calibration for X and T
        ;;----------------------------------------------------

        FOR i = 0, s[1] - 1 DO BEGIN

            FOR j = 0, s[2] - 1 DO BEGIN

                FOR l = 0, s[3]-1 DO BEGIN

                    a = REFORM( stksin[i,j,l,*] )
                    b = Xinv ## a
                    c = Tinv ## b
                    stksout[i,j,l,*] = c

                ENDFOR

           ENDFOR

       ENDFOR

       ;;-------------------------------------------------
       ;; In single-beam there is no left and right
       ;;-------------------------------------------------


     ENDIF

;-------------------------------------------------
;; Done
;-------------------------------------------------

END
