;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_main_polarization
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_main_polarization, i_dir_nb, o_dir_nb, filter, $
                            tfile, xfile1, xfile2, $
                            SCAN_POL = scan_pol, MASK = mask, ROI = roi, WRANGE = wrange, $
                            SINGLE = single, DUAL = dual, STOKES = stokes, $
                            RESIDUAL = residual, EXTERNALC = externalc, EXTERNAL_RESIDUAL_FILE = external_residual_file

;;***************************************************************
;; Data files
;;***************************************************************

     sfiles = file_search( i_dir_nb + filter + '_nb???.sav', COUNT = nnb)

     PRINT,'***************************************'
     PRINT,'Narrowband files found: ', nnb
     PRINT,'***************************************'

;;***************************************************************
;; Modulation
;;***************************************************************

     IF KEYWORD_SET(STOKES) THEN nmod = stokes ELSE nmod = 1

;;***************************************************************
;; Start and end of calibration
;;***************************************************************

     IF KEYWORD_SET(SCAN_POL) THEN BEGIN

        nstart = scan_pol[0]
        nend   = scan_pol[1]

     ENDIF else begin

           nstart = 0
           nend = nnb-1

     ENDELSE

;;***************************************************************
;; Load X-Matrices for both beams
;;***************************************************************

     RESTORE, xfile1, /verb
     xmtxl = xmtx_fit/xmtx_fit[0,0]
     RESTORE, xfile2, /verb
     xmtxr = xmtx_fit/xmtx_fit[0,0]

;;***************************************************************
;; Load T-Matrix parameters
;;***************************************************************

     RESTORE, tfile, /verb
     tparams = tt

;;***************************************************************
;; Calibrate 
;;***************************************************************

     IF KEYWORD_SET(RESIDUAL) THEN BEGIN

        PRINT,'*******************************************************'
        PRINT,'Residual cross-talk correction will be performed'
        PRINT,'*******************************************************'

        IF KEYWORD_SET(EXTERNALC) THEN BEGIN

           PRINT,'*******************************************************'
           PRINT,'External values are used for the correction'
           PRINT,'*******************************************************'

           RESTORE, external_residual_file, /VERB ;;; now variable polfac is known

           polfac = avg(polfactor, 1) 

           init_mask = 0

           FOR i = nstart, nend DO BEGIN

               restore, sfiles[i], /verb

               ;;-------------------------------
               ;; Demodulate, calibrate for XT 
               ;;-------------------------------

               nb_stokes = ibis_calib_pol(nb_data_dstr, info_nb, tparams, xmtxl, xmtxr, filter, $
                                             MASK = mask, INIT_MASK = init_mask, THRESH_MASK = thresh_mask, $
                                             ROI = roi, WRANGE = wrange, $
                                             DUAL = dual, SINGLE = single, $ 
                                             RESIDUAL = residual, EXTERNALC = externalc, $ 
                                             POLFAC = polfac)
         
               ;;-------------------------------
               ;; Save
               ;;-------------------------------


               outfilename = (strmid(sfiles[i], 0, (STRSPLIT(sfiles[i], '.')))) $ 
                                             [N_ELEMENTS(STRSPLIT(sfiles[i], '.'))-1] + 'pc.sav'

               SAVE, /VERB, /COMP, FILENAME = outfilename, $ 
                                   nb_stokes, info_nb


               PRINT,'-------------------------------------------------------'
               PRINT,'Saved : ', outfilename
               PRINT,'-------------------------------------------------------'

           ENDFOR

        ENDIF ELSE BEGIN   ;;; end external

              PRINT,'*********************************************************'
              PRINT,'Values based on each observation are used for correction '
              PRINT,'*********************************************************'

              polfactor = fltarr(3, nend-nstart+1)

              IF KEYWORD_SET(MASK) THEN BEGIN

                 PRINT,'*********************************************************'
                 PRINT,'Using mask procedure'
                 PRINT,'*********************************************************'

                 FOR i = nstart, nend DO BEGIN

                     restore, sfiles[i], /verb

                     ;;-------------------------------
                     ;; Demodulate, calibrate for XT 
                     ;;-------------------------------

                     IF i EQ nstart THEN init_mask = 1 ELSE $ 
                                         init_mask = 0
 
                     nb_stokes = ibis_calib_pol(nb_data_dstr, info_nb, tparams, xmtxl, xmtxr, filter, $
                                                MASK = mask, INIT_MASK = init_mask, THRESH_MASK = thresh_mask, $
                                                ROI = roi, WRANGE = wrange, $
                                                DUAL = dual, SINGLE = single, $ 
                                                RESIDUAL = residual, EXTERNALC = externalc, $ 
                                                POLFAC = polfac)
         
                     polfactor[*,i-nstart] = polfac

                     polfac = 0

                     ;;-------------------------------
                     ;; Save
                     ;;-------------------------------

		     
                     outfilename = (strmid(sfiles[i], 0, (STRSPLIT(sfiles[i], '.')))) $ 
                                             [N_ELEMENTS(STRSPLIT(sfiles[i], '.'))-1] + 'pc.sav'

                     SAVE, /VERB, /COMP, FILENAME = outfilename, $ 
                                         nb_stokes, info_nb


                     PRINT,'-------------------------------------------------------'
                     PRINT,'Saved : ', outfilename
                     PRINT,'-------------------------------------------------------'

                 ENDFOR

             ENDIF  ;;; end mask

              IF KEYWORD_SET(ROI) THEN BEGIN

                 PRINT,'***********************'
                 PRINT,'Using simple region    '
                 PRINT,'***********************'

                 init_mask = 0

                 FOR i = nstart, nend DO BEGIN

                     restore, sfiles[i], /verb

                     ;;-------------------------------
                     ;; Demodulate, calibrate for XT 
                     ;;-------------------------------

                     nb_stokes = ibis_calib_pol(nb_data_dstr, info_nb, tparams, xmtxl, xmtxr, filter, $
                                             MASK = mask, INIT_MASK = init_mask, THRESH_MASK = thresh_mask, $
                                             ROI = roi, WRANGE = wrange, $
                                             DUAL = dual, SINGLE = single, $ 
                                             RESIDUAL = residual, EXTERNALC = externalc, $ 
                                             POLFAC = polfac)

                     polfactor[*,i-nstart] = polfac

                     polfac = 0

                     ;;-------------------------------
                     ;; Save
                     ;;-------------------------------

		     		     
                     outfilename = (strmid(sfiles[i], 0, (STRSPLIT(sfiles[i], '.')))) $ 
                                             [N_ELEMENTS(STRSPLIT(sfiles[i], '.'))-1] + 'pc.sav'

                     SAVE, /VERB, /COMP, FILENAME = outfilename, $ 
                                         nb_stokes, info_nb
		    
                     PRINT,'-------------------------------------------------------'
                     PRINT,'Saved : ', outfilename
                     PRINT,'-------------------------------------------------------'

                  ENDFOR

              ENDIF ;;; end roi

              IF KEYWORD_SET(EXTERNAL_RESIDUAL_FILE) THEN BEGIN

                 PRINT,'****************************************************************************'
                 PRINT,'An external file with residuals will be stored: ', external_residual_file
                 PRINT,'****************************************************************************'

                 SAVE, /VERB, FILENAME = external_residual_file, $
                              polfactor
               
                 PRINT,'-------------------------------------------------------'
                 PRINT,'Saved : ', external_residual_file
                 PRINT,'-------------------------------------------------------'

              ENDIF 

        ENDELSE

     ENDIF  ELSE BEGIN ;;end residual

            PRINT,'*******************************************************'
            PRINT,'NO residual cross-talk calculation will be performed'
            PRINT,'*******************************************************'

            init_mask = 0

            FOR i = nstart, nend DO BEGIN

                restore, sfiles[i], /verb

                ;;-----------------------------------
                ;; Demodulate, calibrate for XT only
                ;;-----------------------------------

                nb_stokes = ibis_calib_pol(nb_data_dstr, info_nb, tparams, xmtxl, xmtxr, filter, $
                                             MASK = mask, INIT_MASK = init_mask, THRESH_MASK = thresh_mask, $
                                             ROI = roi, WRANGE = wrange, $
                                             DUAL = dual, SINGLE = single, $ 
                                             RESIDUAL = residual, EXTERNALC = externalc, $ 
                                             POLFAC = polfac)
         
                ;;-------------------------------
                ;; Save
                ;;-------------------------------

	       
	       		     
                outfilename = (strmid(sfiles[i], 0, (STRSPLIT(sfiles[i], '.')))) $ 
                                             [N_ELEMENTS(STRSPLIT(sfiles[i], '.'))-1] + 'pc.sav'

                SAVE, /VERB, /COMP, FILENAME = outfilename, $ 
                               nb_stokes, info_nb
				   

                PRINT,'-------------------------------------------------------'
                PRINT,'Saved : ', outfilename
                PRINT,'-------------------------------------------------------'

            ENDFOR

     ENDELSE

;;***************************************************************
;; Done
;;***************************************************************

END
