;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_polcalcurve
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;       
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_polcalcurve, indir, outdir, filter_pos, $
                      SAVEFILE1 = savefile1, SAVEFILE2 = savefile2, $
                      SINGLE = single, DUAL = dual
  
;;---------------------------------------------------------------
;; data directories
;;---------------------------------------------------------------

      poldir = file_search(indir + '*_*/')

;;---------------------------------------------------------------
;; get the filenames
;;---------------------------------------------------------------

      polfiles = file_search( poldir + '/*.AutomaticInstrumentalPolarization.fits', COUNT = npolfiles)

      IF npolfiles NE 112 THEN BEGIN

         print,'---------------------------------------------------'
         print,'You do not have a complete set of polarization calibration measurements!'
         print,'---------------------------------------------------'

         GOTO, done

      ENDIF

;;---------------------------------------------------------------
;; get data specifics
;;---------------------------------------------------------------

      sz = read_size(polfiles[0]) 

      info0 = ibis_header(polfiles[0])

      ;; for the polcal measurements all 112 files must have the exact same content

      extension_numbers = ibis_file_structure(polfiles[0], filter_pos, INFO = www)
      extensions = extension_numbers.full            ;;;; polcals are done in full Stokes mode.

      nrepeat  = www.nrepeat_full[0]                 ;;;; = 1 : there are no repeats allowed for polcals
      nroption = unique_elements(www.nroption_full)  ;;;; constant for all wavelength points.
      ni       = n_elements(extensions) / nrepeat
      nlam     = n_elements(www.nroption_full)  
      nmod     = ni / nlam

      print,'---------------------------------------------------'
      print,'Number of wavelength points : ', nlam
      print,'Wavelength points : ', info0.wave[extensions-1]
      print,'Number of modulation states : ', nmod
      print,'---------------------------------------------------'

;;---------------------------------------------------------------
;; define arrays
;;---------------------------------------------------------------

      info_pol = ibis_create_struct(ni, NFILES = npolfiles)

      nb_polcal = fltarr(npolfiles, ni, 2)

;;---------------------------------------------------------------
;; find aperture masks
;;---------------------------------------------------------------

      FITS_OPEN, polfiles[0], tmp_nb
      FITS_READ, tmp_nb, data, header, exten_no = extensions[0], /NO_PDU

      Nx    = sz[0]
      Ny    = sz[1]

      WINDOW, 0, xs = sz[0], ys = sz[1]

      ;; tvscl, data

      ;; PRINT,'--------------------------------------------------------------------'
      ;; PRINT,'Determine lower left corner of ROI in left beam (click on lower left pixel position)'
      ;; winplot
      ;; xy
      ;; PRINT & PAUSE,'Enter the pixel positions for x and y with spacing', /nopause, ans = ans
      ;; ans = fix(get_words(ans))
      ;; xa1 = ans[0]
      ;; ya1 = ans[1]
      ;; PRINT,'Determine upper right corner of ROI in left beam (click on upper right pixel position)'
      ;; xy
      ;; PRINT & PAUSE,'Enter the pixel positions for x and y with spacing', /nopause, ans = ans
      ;; ans = fix(get_words(ans))
      ;; xa2 = ans[0]
      ;; ya2 = ans[1]
      ;; PRINT,'Determine lower left corner of ROI in right beam (click on lower left pixel position)'
      ;; xy
      ;; PRINT & PAUSE,'Enter the pixel positions for x and y with spacing', /nopause, ans = ans
      ;; ans = fix(get_words(ans))
      ;; xb1 = ans[0]
      ;; yb1 = ans[1]
      ;; PRINT,'Determine upper right corner of ROI in right beam (click on upper right pixel position)'
      ;; xy
      ;; PRINT & PAUSE,'Enter the pixel positions for x and y with spacing', /nopause, ans = ans
      ;; ans = fix(get_words(ans))
      ;; xb2 = ans[0]
      ;; yb2 = ans[1]
      ;; PRINT,'--------------------------------------------------------------------'

      aps  = ibis_get_aperture(data)
;      idxa  = WHERE(aps GE 1, Num)
;      idxb  = WHERE(aps GE 1 AND aps LT 2, Num)

      idxl  = where(aps[0:Nx/2-1,*] ge 1)
;      idxlx = reform(idxl[*,0])
;      idxly = reform(idxl[*,1])
;      Numl = n_elements(idxlx)

      idxr  = where(aps[Nx/2:Nx-1,*] ge 1)
;      idxrx = reform(idxr[*,0])
;      idxry = reform(idxr[*,1])
;      Numr = n_elements(idxrx)

      FITS_CLOSE, tmp_nb

      wdelete

;;---------------------------------------------------------------
;; loop through polarization calibration files
;;---------------------------------------------------------------

      FOR n = 0, npolfiles-1 DO BEGIN

          print,'---------------------------------------------------'
          print,'Polarization calibration file : ', polfiles[n]

          FITS_OPEN, polfiles[n], tmp_nb

          FOR i = 0L, ni-1 DO BEGIN

              FITS_READ, tmp_nb, data, header, exten_no = extensions[i], /NO_PDU
              nb_polcal[n,i,0] = avg((reform(data[0:Nx/2-1,*]))[idxl]) ;;;avg(data[xa1:xa2,ya1:ya2]) ;;; data[271,381] ;; avg(data[idxa])
              nb_polcal[n,i,1] = avg((reform(data[Nx/2:Nx-1,*]))[idxr]) ;;;avg(data[xb1:xb2,yb1:yb2]) ;;; data[751,378] ;; avg(data[idxb])
              info_pol = ibis_info(header, info_pol, polfiles[n], extensions[i], i, NFILES = n)

          ENDFOR

          FITS_CLOSE, tmp_nb

      ENDFOR
          
          print,'---------------------------------------------------'

;;---------------------------------------------------------------
;; reform and average over wavelength points
;;---------------------------------------------------------------

      modulation = (reform(info_pol.stokes[*,0], nmod, nlam))[0:nmod-1]

      nb_polcal = avg(reform(nb_polcal, npolfiles, nmod, nlam, 2), 2)

      pc_azim   = avg(reform(info_pol.dst_az    , nmod, nlam, npolfiles),1)
      pc_elev   = avg(reform(info_pol.dst_el    , nmod, nlam, npolfiles),1)
      pc_table  = avg(reform(info_pol.dst_tbl   , nmod, nlam, npolfiles),1)
      pc_llevel = avg(reform(info_pol.dst_llevel, nmod, nlam, npolfiles),1)

;;---------------------------------------------------------------
;; correct for lightlevel variation
;;---------------------------------------------------------------

      lightcurve = reform(pc_llevel[0,*])
      lightcurve = lightcurve / avg(lightcurve)
      coef = poly_fit(findgen(npolfiles), lightcurve, 2, yfit = yy)

      WINDOW, 0, xs = 900, ys = 300, title='Light curve (solid) and parabolic fit (dotted)'
      plot, lightcurve, xtitle = 'measurements', ytitle='DST guider light level'
      oplot, yy, lines=1

      WINDOW, 0, xs = 900, ys = 300, title='Deviation from parabolic fit'
      plot, lightcurve / yy, xtitle = 'measurements', ytitle='normalized light level'

      PRINT & PAUSE,'If the lightcurve has dips that are >~ 5-10% then a manual correction needs to be done (return to continue)', /nopause

      nbpolcal = nb_polcal

      FOR i = 0, nmod - 1 DO BEGIN

          nbpolcal[*, i, 0] = REFORM( nb_polcal[*, i, 0] ) / lightcurve
          nbpolcal[*, i, 1] = REFORM( nb_polcal[*, i, 1] ) / lightcurve
 
      ENDFOR

;;---------------------------------------------------------------
;; display modulated calibration curves before/after correction
;;---------------------------------------------------------------

      window, 0, xs = 800, ys = 900, title = 'Before (dotted) and after (solid) lightcurve correction'
      !p.multi = [0,1,6]
      plot, nbpolcal[*,0,0]
      oplot, nb_polcal[*,0,0],lines=1
      plot, nbpolcal[*,1,0]
      oplot, nb_polcal[*,1,0],lines=1
      plot, nbpolcal[*,2,0]
      oplot, nb_polcal[*,2,0],lines=1
      plot, nbpolcal[*,3,0]
      oplot, nb_polcal[*,3,0],lines=1
      plot, nbpolcal[*,4,0]
      oplot, nb_polcal[*,4,0],lines=1
      plot, nbpolcal[*,5,0]
      oplot, nb_polcal[*,5,0],lines=1

;;---------------------------------------------------------------
;; demodulate calibration curves
;;---------------------------------------------------------------

      ibis_demodulate_polcal, nbpolcal, modulation, stokesvect, SINGLE = single, DUAL = dual

      telparam_azi    = KONT( pc_azim )
      telparam_elev   = KONT( pc_elev )
      telparam_table  = KONT( pc_table )
      telparam_llevel = KONT( pc_llevel )

;;---------------------------------------------------------------
;; display demodulated curves
;;---------------------------------------------------------------

      window, 2, xs = 800, ys = 900, title = 'After de-modulation'
      !p.multi = [0,1,4]
      plot, stokesvect[*,0,0]
      plot, stokesvect[*,1,0]
      plot, stokesvect[*,2,0]
      plot, stokesvect[*,3,0]

      !p.multi = 0

;;---------------------------------------------------------------
;; gather calibration unit orientations
;;---------------------------------------------------------------

      tmp = strtrim(reform(info_pol.pt4_pstg[0,*]))
      index1 = where(tmp eq 'IN')
      index2 = where(tmp eq 'OUT')
      in_beam_lin_pol = intarr(npolfiles)
      in_beam_lin_pol[index1] = 1
      in_beam_lin_pol[index2] = 0

      tmp = strtrim(reform(info_pol.pt4_rstg[0,*]))
      index1 = where(tmp eq 'IN')
      index2 = where(tmp eq 'OUT')
      in_beam_retarder = intarr(npolfiles)
      in_beam_retarder[index1] = 1
      in_beam_retarder[index2] = 0

      tmp = strtrim(reform(info_pol.pt4_fs[0,*]))
;tmp([1,26,29,54,57,82,85,110]) = 'DARK'
      index1 = where(tmp eq 'DARK')
      index2 = where(tmp ne 'DARK')
      in_beam_dark_slide = intarr(npolfiles)
      in_beam_dark_slide[index1] = 1  
      in_beam_dark_slide[index2] = 0

      cal_settings_lin_pol  = float(round(reform(info_pol.pt4_pol[0,*])))
      cal_settings_retarder = float(round(reform(info_pol.pt4_ret[0,*])))

;;---------------------------------------------------------------
;; Save output
;;---------------------------------------------------------------
  
      IF KEYWORD_SET(SAVEFILE1) THEN BEGIN

         print,'---------------------------------------------------'
         print,'Save : ', outdir + savefile1
         print,'---------------------------------------------------'

         nb_polcal  = nbpolcal
         SAVE, /VERB, /COMP, FILENAME = outdir + savefile1, $
               modulation, nb_polcal, pc_azim, pc_elev, pc_table, pc_llevel 

      ENDIF ELSE BEGIN

            PRINT,'No modulated polarization curves saved!'

      ENDELSE

      IF KEYWORD_SET(SAVEFILE2) THEN BEGIN

         print,'---------------------------------------------------'
         print,'Save : ', outdir + savefile2
         print,'---------------------------------------------------'

         SAVE, /VERB, /COMP, FILENAME = outdir + savefile2, $
               stokesvect, telparam_azi, telparam_elev, telparam_table, telparam_llevel, $
               in_beam_lin_pol, in_beam_retarder, in_beam_dark_slide, cal_settings_lin_pol, $
               cal_settings_retarder

      ENDIF ELSE BEGIN

            PRINT,'No demodulated polarization curves saved!'

      ENDELSE

;;---------------------------------------------------------------
;; Done
;;---------------------------------------------------------------

done:

END
