;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_residual, stksin, stksout, $
                   MASK = mask, ROI = roi, WRANGE = wrange, $ 
                   POLFAC = polfac

;-------------------------------------------------
;; Constants, variables, settings
;-------------------------------------------------

     c1 = wrange[0]
     c2 = wrange[1]
     nc = FLOAT(c2-c1+1)
 
;-------------------------------------------------
;; Form Stokes data arrays and continuum maps
;-------------------------------------------------

     ii = REFORM( stksin[*,*,*,0] )
     qq = REFORM( stksin[*,*,*,1] )
     uu = REFORM( stksin[*,*,*,2] )
     vv = REFORM( stksin[*,*,*,3] )

     vcont = TOTAL( vv[*,*,c1:c2], 3 ) / nc
     icont = TOTAL( ii[*,*,c1:c2], 3 ) / nc
     qcont = TOTAL( qq[*,*,c1:c2], 3 ) / nc
     ucont = TOTAL( uu[*,*,c1:c2], 3 ) / nc

;;-------------------------------------------------
;; Polarization factors known or not?
;;-------------------------------------------------

     IF KEYWORD_SET(POLFAC) THEN BEGIN 

        ;;-------------------------------------------------
        ;; Correct for crosstalk I-->Q,U,V 
        ;;-------------------------------------------------

        print,'-------------------------------------------------'
        print,'Using known polarization factors for correction'
        print,'-------------------------------------------------'

        vmap = vv
        qmap = qq
        umap = uu
     
        polfacv = reform(polfac[0])
        polfacq = reform(polfac[1])
        polfacu = reform(polfac[2])

        s = SIZE(vmap)             

        FOR j = 0,s[3]-1 DO BEGIN
         
            vmap[*,*,j] = vv[*,*,j] - (polfacv * ii[*,*,j])
            qmap[*,*,j] = qq[*,*,j] - (polfacq * ii[*,*,j])
            umap[*,*,j] = uu[*,*,j] - (polfacu * ii[*,*,j])
             
        ENDFOR 

     ENDIF ELSE BEGIN

           ;;-------------------------------------------------
           ;; Calculate I->QUV cross talk fractions
           ;;-------------------------------------------------

           IF KEYWORD_SET(ROI) THEN BEGIN

              xa = roi[0] & xb = roi[2]
              ya = roi[1] & yb = roi[3]

              polfacv = AVG( (vcont/icont)[xa:xb,ya:yb] )
              polfacq = AVG( (qcont/icont)[xa:xb,ya:yb] )
              polfacu = AVG( (ucont/icont)[xa:xb,ya:yb] )

           ENDIF

           IF KEYWORD_SET(MASK) THEN BEGIN

              index  = where(mask eq 1)
              polfacv = AVG( (vcont/icont)[index] )
              polfacq = AVG( (qcont/icont)[index] )
              polfacu = AVG( (ucont/icont)[index] )

           ENDIF

           ;;-------------------------------------------------
           ;; Correct for crosstalk I-->Q,U,V 
           ;;-------------------------------------------------

           vmap = vv
           qmap = qq
           umap = uu
     
           s = SIZE(vmap)             

           FOR j = 0,s[3]-1 DO BEGIN
         
               vmap[*,*,j] = vv[*,*,j] - (polfacv * ii[*,*,j])
               qmap[*,*,j] = qq[*,*,j] - (polfacq * ii[*,*,j])
               umap[*,*,j] = uu[*,*,j] - (polfacu * ii[*,*,j])
             
           ENDFOR 

           polfac = [polfacv, polfacq, polfacu]

     ENDELSE

;-------------------------------------------------
;; Form output array
;-------------------------------------------------

     stksout = stksin

     stksout[*,*,*,0] = ii
     stksout[*,*,*,1] = qmap
     stksout[*,*,*,2] = umap
     stksout[*,*,*,3] = vmap

;-------------------------------------------------
;; Done
;-------------------------------------------------

END
