;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_parameter
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_parameter, tmpa, tmpb, boxsz

     window,0, xs = (size(tmpb))[1], ys = (size(tmpb))[2]
     tvscl,tmpb
     xycmean=ibis_xy_4pnts_corr(tmpb, 'Broadband', KERN=boxsz)

     window,2, xs = (size(tmpa))[1], ys = (size(tmpa))[2]
     tvscl,tmpa
     xyfamean=ibis_xy_4pnts_corr(tmpa, 'Narrowband', KERN=boxsz)

     PRINT,'--------------------------------------------------------------------'
     PRINT,'The distance between each grid lines is 1 mm'
     PRINT & PAUSE,'Enter the distance between selected points 1 and 2 in mm (and hit return).', /nopause, ans = ans
     ans = fix(get_words(ans))
     distx = ans[0]
     PRINT & PAUSE,'Enter the distance between selected points 2 and 3 in mm (and hit return).', /nopause, ans = ans
     ans = fix(get_words(ans))
     disty = ans[0]
     PRINT,'--------------------------------------------------------------------'

     xcc1=xycmean(0,0) & xcc2=xycmean(1,0) & xcc3=xycmean(2,0) &xcc4=xycmean(3,0)
     ycc1=xycmean(0,1) & ycc2=xycmean(1,1) & ycc3=xycmean(2,1) &ycc4=xycmean(3,1)

     xffa1=xyfamean(0,0) & xffa2=xyfamean(1,0) & xffa3=xyfamean(2,0) &xffa4=xyfamean(3,0)
     yffa1=xyfamean(0,1) & yffa2=xyfamean(1,1) & yffa3=xyfamean(2,1) &yffa4=xyfamean(3,1)
;
;  Relative distances of the four continuum image points:

     cc12=sqrt((xcc2-xcc1)^2+(ycc2-ycc1)^2)  
     cc34=sqrt((xcc4-xcc3)^2+(ycc4-ycc3)^2)

     cc23=sqrt((xcc3-xcc2)^2+(ycc3-ycc2)^2) 
     cc41=sqrt((xcc1-xcc4)^2+(ycc1-ycc4)^2) 

     cc13=sqrt((xcc3-xcc1)^2+(ycc3-ycc1)^2)
     cc24=sqrt((xcc4-xcc2)^2+(ycc4-ycc2)^2)
;
;  Relative distances of the four filter image points:

     ffa12=sqrt((xffa2-xffa1)^2+(yffa2-yffa1)^2) 
     ffa34=sqrt((xffa4-xffa3)^2+(yffa4-yffa3)^2) 

     ffa23=sqrt((xffa3-xffa2)^2+(yffa3-yffa2)^2) 
     ffa41=sqrt((xffa1-xffa4)^2+(yffa1-yffa4)^2) 

     ffa13=sqrt((xffa3-xffa1)^2+(yffa3-yffa1)^2)
     ffa24=sqrt((xffa4-xffa2)^2+(yffa4-yffa2)^2)

;
; Image scale in both directions 

     bb_scale_x = 3.76/(avg([cc12,cc34])) * distx
     bb_scale_y = 3.76/(avg([cc23,cc41])) * disty

     nb_scale_x = 3.76/(avg([ffa12,ffa34])) * distx
     nb_scale_y = 3.76/(avg([ffa23,ffa41])) * disty

     print,'----------------------------------------------------------------'
     print,'Image scale in broadband [x,y] : ', bb_scale_x, bb_scale_y
     print,'Image scale in narrowband [x,y] : ', nb_scale_x, nb_scale_y
     print,'----------------------------------------------------------------'

;
;  Scaling factor = mean of filter-distances / continuum-distances:

     magn1_c=(ffa12/cc12+ffa23/cc23+ffa34/cc34+ffa41/cc41+ffa13/cc13+ffa24/cc24)/6.
     magn1_cx=(ffa12/cc12+ffa34/cc34)/2.
     magn1_cy=(ffa23/cc23+ffa41/cc41)/2.
;
;  Scaling factor = mean of continuum-distances / filter-distances:

     magnc_1=(cc12/ffa12+cc23/ffa23+cc34/ffa34+cc41/ffa41+cc13/ffa13+cc24/ffa24)/6.
     magnc_1x=(cc12/ffa12+cc34/ffa34)/2.
     magnc_1y=(cc23/ffa23+cc41/ffa41)/2.

;
;  Angles defined by the four target points, continuum-image:

     ccalf12=atan((ycc2-ycc1),(xcc2-xcc1))
     ccalf23=atan((ycc3-ycc2),(xcc3-xcc2))
     ccalf34=atan((ycc4-ycc3),(xcc4-xcc3))
     ccalf41=atan((ycc1-ycc4),(xcc1-xcc4))
     ccalf13=atan((ycc3-ycc1),(xcc3-xcc1))
     ccalf24=atan((ycc4-ycc2),(xcc4-xcc2))
;
;  Angles defined by the four target points, filter-image:

     ffaalf12=atan((yffa2-yffa1),(xffa2-xffa1))
     ffaalf23=atan((yffa3-yffa2),(xffa3-xffa2))
     ffaalf34=atan((yffa4-yffa3),(xffa4-xffa3))
     ffaalf41=atan((yffa1-yffa4),(xffa1-xffa4))
     ffaalf13=atan((yffa3-yffa1),(xffa3-xffa1))
     ffaalf24=atan((yffa4-yffa2),(xffa4-xffa2))

; calculate angles
			    
   if ccalf12 gt 0.75*!pi and ffaalf12 lt -0.75*!pi then ffaalf12=ffaalf12+2*!pi
   if ccalf12 lt -0.75*!pi and ffaalf12 gt 0.75*!pi then ccalf12=ccalf12+2*!pi
   if ccalf23 gt 0.75*!pi and ffaalf23 lt -0.75*!pi then ffaalf23=ffaalf23+2*!pi 
   if ccalf23 lt -0.75*!pi and ffaalf23 gt 0.75*!pi then ccalf23=ccalf23+2*!pi
   if ccalf34 gt 0.75*!pi and ffaalf34 lt -0.75*!pi then ffaalf34=ffaalf34+2*!pi
   if ccalf34 lt -0.75*!pi and ffaalf34 gt 0.75*!pi then ccalf34=ccalf34+2*!pi
   if ccalf41 gt 0.75*!pi and ffaalf41 lt -0.75*!pi then ffaalf41=ffaalf41+2*!pi
   if ccalf41 lt -0.75*!pi and ffaalf41 gt 0.75*!pi then ccalf41=ccalf41+2*!pi
   if ccalf13 gt 0.75*!pi and ffaalf13 lt -0.75*!pi then ffaalf13=ffaalf13+2*!pi
   if ccalf13 lt -0.75*!pi and ffaalf13 gt 0.75*!pi then ccalf13=ccalf13+2*!pi
   if ccalf24 gt 0.75*!pi and ffaalf24 lt -0.75*!pi then ffaalf24=ffaalf24+2*!pi
   if ccalf24 lt -0.75*!pi and ffaalf24 gt 0.75*!pi then ccalf24=ccalf24+2*!pi

   dalf1_c=-((ffaalf12-ccalf12)+(ffaalf23-ccalf23)+(ffaalf34-ccalf34)+ $
           (ffaalf41-ccalf41)+(ffaalf13-ccalf13)+(ffaalf24-ccalf24))*!radeg/6.
				    
   if ffaalf12 gt 0.75*!pi and ccalf12 lt -0.75*!pi then ccalf12=ccalf12+2*!pi
   if ffaalf12 lt -0.75*!pi and ccalf12 gt 0.75*!pi then ffaalf12=ffaalf12+2*!pi
   if ffaalf23 gt 0.75*!pi and ccalf23 lt -0.75*!pi then ccalf23=ccalf23+2*!pi 
   if ffaalf23 lt -0.75*!pi and ccalf23 gt 0.75*!pi then ffaalf23=ffaalf23+2*!pi
   if ffaalf34 gt 0.75*!pi and ccalf34 lt -0.75*!pi then ccalf34=ccalf34+2*!pi
   if ffaalf34 lt -0.75*!pi and ccalf34 gt 0.75*!pi then ffaalf34=ffaalf34+2*!pi
   if ffaalf41 gt 0.75*!pi and ccalf41 lt -0.75*!pi then ccalf41=ccalf41+2*!pi
   if ffaalf41 lt -0.75*!pi and ccalf41 gt 0.75*!pi then ffaalf41=ffaalf41+2*!pi
   if ffaalf13 gt 0.75*!pi and ccalf13 lt -0.75*!pi then ccalf13=ccalf13+2*!pi
   if ffaalf13 lt -0.75*!pi and ccalf13 gt 0.75*!pi then ffaalf13=ffaalf13+2*!pi
   if ffaalf24 gt 0.75*!pi and ccalf24 lt -0.75*!pi then ccalf24=ccalf24+2*!pi
   if ffaalf24 lt -0.75*!pi and ccalf24 gt 0.75*!pi then ffaalf24=ffaalf24+2*!pi

   dalfc_1=-((ccalf12-ffaalf12)+(ccalf23-ffaalf23)+(ccalf34-ffaalf34)+ $
           (ccalf41-ffaalf41)+(ccalf13-ffaalf13)+(ccalf24-ffaalf24))*!radeg/6.
	    
;; apply to whitelight channel, determine offset and display result

   f1 = magn1_cx
   f2 = magn1_cy
   r  = dalf1_c

   tmp = congrid(tmpb, f1*(size(tmpb))[1], f2*(size(tmpb))[2], cubic=-0.5)
   tmp = rot(tmp,r,cubic=-0.5)
   tmp0 = ibis_set(tmp, (size(tmpa))[1], (size(tmpa))[2])
   dxy = shc(tmpa,tmp0,/interpolate)
  
   tmp0n = shift_bicub(tmp0,dxy[0],dxy[1])
 
   tvscl, tmpa - tmp0n

   sfl = [dxy[0],dxy[1]]

   alignment_values = create_struct('r', 0.0, 'f1', 0.0, 'f2', 0.0, 'sfl', [0.0,0.0])
   alignment_values.r   = r
   alignment_values.f1  = f1
   alignment_values.f2  = f2
   alignment_values.sfl = sfl

RETURN, alignment_values

END
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_align
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Jun-22, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_align, sdir, wdir, nstart, nend, ORIENTATION = orientation, METHOD = method, $ 
                SAVEFILE1 = savefile1, SAVEFILE2 = savefile2, SAVEFILE3 = savefile3

     sgfiles = file_search( sdir + '*.GridImages.fits', COUNT = ns)
     wgfiles = file_search( wdir + '*.GridImages.fits', COUNT = nw)

     PRINT,'------------------------------------------'
     PRINT,'Line-grid narrowband files found: ', ns
     PRINT,'Line-grid broadband files found: ', nw
     PRINT,'------------------------------------------'

     ni = nend - nstart + 1
     sz = read_size(sgfiles[0])
     nb_grid = fltarr(sz[0],sz[1],ni*ns)
     bb_grid = fltarr(sz[0],sz[1],ni*nw)

     l = 0

     align = fltarr(5,ns*ni)

     FOR k = 0, ns-1 DO BEGIN

         print,'Narrowband grid file : ', sgfiles[k]
         print,'Broadband grid file : ', wgfiles[k]

         FOR i = 1L, ni DO BEGIN

                nb_grid[*,*,l] = READFITS(sgfiles[k], EXT = nstart+i-1, header, /SILENT)
                IF orientation EQ 'default' THEN bb_grid[*,*,l] = REVERSE(ROTATE(READFITS(wgfiles[k], EXT = nstart+i-1, header, /SILENT),1))

                l = l+1

         ENDFOR

     ENDFOR

     save, nb_grid, filename = savefile2
     save, bb_grid, filename = savefile3

     PRINT & PAUSE,'Enter image number in grid sequence (def = use 0)', /nopause, ans = ans

     IF ans EQ '' THEN BEGIN

	PRINT,'Process zero-th image in grid sequence'
       
        index = 0

     ENDIF ELSE BEGIN

           index = fix(get_words(ans))
   
     ENDELSE
  
     nb_tmp = reform(nb_grid[*,*,index])
     bb_tmp = reform(bb_grid[*,*,index])

     if keyword_set(METHOD) eq 1 and method eq 'CORR' THEN BEGIN

        boxsize = 20
        parameter = ibis_parameter(nb_tmp, bb_tmp, boxsize)

        print,'----------------------------------------------------------------'
        print,'Relative rotation: ', parameter.r
        print,'Relative scale in x: ', parameter.f1
        print,'Relative scale in y: ', parameter.f2
        print,'Relative offset: ', parameter.sfl
        print,'----------------------------------------------------------------'

     ENDIF

     if keyword_set(METHOD) eq 1 and method eq 'HOUGH' THEN BEGIN

        print, 'hahaha' 

     ENDIF

    save, parameter, filename = savefile1


END

