;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_parameter
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
FUNCTION ibis_calib, nb_data, bb_data, gain, dark, info_s, info_w, dstr_idl, align_parameter, $ 
                     bshift, aps, Npol, mwld, shft_tot, bshift_dstr, $ 
                     POL = pol, DUAL = dual, SINGLE = single, $ 
                     REF_FILE = ref_file, ORIENTATION = orientation


  r   = align_parameter.r
  f1  = align_parameter.f1
  f2  = align_parameter.f2
  sfl = align_parameter.sfl

  dim   = size(nb_data)
  Ncol  = dim[1]
  Nrow  = dim[2]

  DS_KERNEL = (dstr_idl.kernel)
  DT_DETR   = (dstr_idl.detrend)
  METHOD    = (dstr_idl.method)
  REGION    = (dstr_idl.region)
  AVG_IMG   = (dstr_idl.avg_img)

  Nwave = dim[3]/Npol
  modulation = (info_s.stokes)[0:Npol-1]
 
  print,'---------------------------------------------------'
  print,'Ok, here the dimensions:                           '       
  print,'Spatial dimensions: ', Ncol, Nrow                   
  print,'Wavelength points: ', Nwave
  print,'Number of modulation states : ', Npol
  print,'Modulation scheme : ', modulation
  print,'Does that seem right ?'
  print,'---------------------------------------------------'

;;***************************************************************
;; Determine mask indices
;;***************************************************************

  mask = fix(aps)
  index_in = WHERE(mask EQ 1)
  index_out = WHERE(mask EQ 0)

;;***************************************************************
;; Get index for ordering wavelengths
;;***************************************************************

  wtmp = REFORM((info_s.grid))
  wtmp = wtmp[uniq(wtmp)]
  index_wl = sort(wtmp)
  index_ff = sort(index_wl)
  wscale = wtmp[index_wl]

;;#################################################################
;;#################################################################
;; 
;; POLARIMETRY CASE
;;
;;#################################################################
;;#################################################################

  IF KEYWORD_SET(POL) THEN BEGIN

     IF KEYWORD_SET(DUAL) THEN BEGIN

         print,'---------------------------------------------------'
         print,'Ok, We are in Polarimetric dual-beam Mode                    '
         print,'---------------------------------------------------'

        ;;#################################################################
        ;; We used to do a lightlevel correction at this point but we
        ;; will not do this anymore until the llevel input extracted
        ;; from the DST guider light level is updated
        ;; with an appropriate cadence. 
        ;;#################################################################

        ;;#################################################################
        ;; Telescope geometry is now available with the file
        ;;#################################################################

        ;;#################################################################
        ;; Determine alignment between left and right beam
        ;; before gain correction
        ;;#################################################################

        print,'-----------------------------------------'
        print,'Alignment between left and right beam '
        print,'-----------------------------------------'

        dxy = fltarr(2)
  
        FOR i=0,dim[3]-1 DO BEGIN
            dxy += shc(nb_data[0:Ncol/2-1, *, i], nb_data[Ncol/2:Ncol-1, *, i], /interp)
        ENDFOR
        dxy /= dim[3]

        ;;#################################################################
        ;; Flatfield correction for narrowband
        ;;#################################################################

        ;; Apply gain table for each wavelength separately.
 
        print,'--------------------------------------'
        print,'Gain table correction for narrowband  '
        print,'--------------------------------------'

        mwldn = 0. * bb_data
        nbdwc = 0. * nb_data

        nbdark = avg(dark,2)

        l = 0

        FOR n=0, Nwave-1 DO BEGIN  
            FOR i=0, Npol-1 DO BEGIN
               mwldn[*,*,l] = bb_data[*,*,l] ; / lightlevel[l]
               nbdwc[*, *, l] = ((nb_data[*, *, l] - dark) * gain[*, *, index_ff[n], i]) ; / lightlevel[l]
               l += 1
            ENDFOR
        ENDFOR

        mwld = mwldn
        mwldn = 0

        ;;#################################################################
        ;; Alignment between left and right channel
        ;;#################################################################

        ;;
        ;; shift right subaperture on left (also apply mask and offset)
        ;;

        print,'-----------------------------------------'
        print,'Alignment between left and right channel '
        print,'-----------------------------------------'

        FOR i=0, dim[3]-1 DO BEGIN
            tmp = shift_bicub(nbdwc[Ncol/2:Ncol-1, *, i], dxy[0], dxy[1])
            nbdwc[Ncol/2:Ncol-1, *, i] = tmp
        ENDFOR

        aps_mod = aps
        tmp = shift_bicub(float(aps[Ncol/2:Ncol-1, *]), dxy[0], dxy[1])
        aps_mod[Ncol/2:Ncol-1, *] = FIX(ROUND(tmp))

        bshift_mod = bshift
        tmp = shift_bicub(bshift[Ncol/2:Ncol-1, *], dxy[0], dxy[1])
        bshift_mod[Ncol/2:Ncol-1, *] = tmp
        
        ;;#################################################################
        ;; Resample the arrays
        ;;#################################################################

        nbsl = nbdwc[0:Ncol/2-1, 0:Nrow-1, *]
        nbsr = nbdwc[Ncol/2:Ncol-1, 0:Nrow-1, *]

        bsds = rebin(bshift_mod, Ncol, Nrow, Npol*Nwave)
        blsl = bsds[0:Ncol/2-1, 0:Nrow-1, *]
        blsr = bsds[Ncol/2:Ncol-1, 0:Nrow-1, *]

        print,'-----------------------------------------'
        print,'Offset mask size: ', size(bshift)
        print,'-----------------------------------------'

        s = size(mwld)

        ;;#################################################################
        ;; Do the destretch with IDL code
        ;;#################################################################

        print,'-----------------------------------------'
        print,'Destretch starts now .....               '
        print,'-----------------------------------------'

        print,'-----------------------------------------'
        print,'Destretch mode: IDL-code                 '
        print,'Method : ', method
        print,'Kernel : ', DS_KERNEL
        print,'-----------------------------------------'

        dummy = reg(findgen(100,100), findgen(100,100), bytarr(12,12))
        forward_function doreg, mkcps, repair

        ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
        ;; Destretch whitelight
        ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

        IF method EQ 'AVG' THEN BEGIN
           ref = avg(mwld,2)
           mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
        ENDIF

        IF method EQ 'SEQ' THEN BEGIN
           mwld2 = destretch_ibis(AVG_IMG[0], mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
        ENDIF

        IF method EQ 'EXTERNAL' THEN BEGIN

           restore, ref_file, /verb

           IF (SIZE(image))[0] EQ 2 THEN BEGIN 
 
              tmp = image

              sbbi = SIZE(tmp)

              IF orientation eq 'default' then tmp = reverse(rotate(tmp,1)) 

              tmp = CONGRID(tmp, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
              tmp = rot(tmp,r,cubic=-0.5)
              tmp_n = ibis_set(tmp,sbbi[1],sbbi[2])
              tmp_n = shift_bicub(tmp_n,sfl[0],sfl[1])

              image = tmp_n

           ENDIF

           IF (SIZE(image))[0] EQ 3 THEN BEGIN 

              print,'-------------------------------------------------------------------------'
              print,' More than one reconstruction found for this scan: ', (SIZE(image))[3]
              print,'-------------------------------------------------------------------------'

              images = fltarr(Ncol,Nrow, (SIZE(image))[3])

              FOR i = 0, (SIZE(image))[3]-1 DO BEGIN

                  tmp = reform(image[*,*,i])

                  sbbi = SIZE(tmp)

                  IF orientation eq 'default' then tmp = reverse(rotate(tmp,1)) 

                  tmp = CONGRID(tmp, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
                  tmp = rot(tmp,r,cubic=-0.5)
                  tmp_n = ibis_set(tmp,sbbi[1],sbbi[2])
                  tmp_n = shift_bicub(tmp_n,sfl[0],sfl[1])
                
                  ;;
                  ;; align references ;;;; this is the part that
                  ;; becomes obsolete.
                  ;;

                  IF i EQ 0 THEN ref = tmp_n
                     
                     dxy = shc(ref[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], $ 
                                   tmp_n[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], /interpolate)
                     tmp = shift_bicub(tmp_n, dxy[0], dxy[1])

                     images[*,*,i] = tmp_n

              ENDFOR

              image = images

           ENDIF

           ;;
           ;; ref in this case can be one or >1 image!
           ;;

           ref = image
           mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)

        ENDIF

        mwld = mwld2

        ;;#################################################################
        ;; Destretch narrowband and blueshift
        ;;#################################################################

        print,'-------------------------------------------------------'
        print,'Destretch of whitelight done .....                     '
        print,'Now we destretch the narrowband and the blueshift map  '
        print,'-------------------------------------------------------'

        IF N_ELEMENTS(DS_KERNEL) EQ 1 THEN BEGIN
              
           print,'Align shift :', shft_tot[*, 0, 0, 0]

           nb = 0. * nbdwc
           FOR i = 0,dim[3]-1 DO BEGIN
               nb[*, *, i] = shift_bicub(nbdwc[*, *, i], shft_tot[0, 0, 0, i], shft_tot[1, 0, 0, i])
           ENDFOR

           blds = rebin(bshift, Ncol, Nrow, Npol*Nwave)
           bshift_dtsr = 0. * blds
           FOR i = 0,dim[3]-1 DO BEGIN
               bshift_dstr[*, *, i] = shift_bicub(blds[*, *, i], shft_tot[0, 0, 0, i], shft_tot[1, 0, 0, i])
           ENDFOR

        ENDIF ELSE BEGIN

              ;;
              ;; destretch narrowband left and right
              ;;

              FOR i = 0,dim[3]-1 DO BEGIN
                  nbsl[*, *, i] = doreg(nbsl[*, *, i], grid, shft_tot[*, *, *, i])
                  nbsr[*, *, i] = doreg(nbsr[*, *, i], grid, shft_tot[*, *, *, i])
              ENDFOR

              ;;
              ;; right and left blueshift
              ;;

              FOR i=0,dim[3]-1 DO BEGIN
                  blsl[*, *, i] = doreg(blsl[*, *, i], grid, shft_tot[*, *, *, i])
                  blsr[*, *, i] = doreg(blsr[*, *, i], grid, shft_tot[*, *, *, i])
              ENDFOR

        ENDELSE

        ;;#################################################################
        ;; Perform blueshift correction
        ;;#################################################################

        print,'-------------------------------------------------------'
        print,'Now we correct the narrowband for the blueshift        '
        print,'-------------------------------------------------------'

        nbsl = REFORM(nbsl, Ncol/2, Nrow, Npol, Nwave)
        nbsr = REFORM(nbsr, Ncol/2, Nrow, Npol, Nwave)

        blsl = REFORM(blsl, Ncol/2, Nrow, Npol, Nwave)
        blsr = REFORM(blsr, Ncol/2, Nrow, Npol, Nwave)

;       wtmp = REFORM(exposures.wavelength_offset)
;       wtmp = wtmp[uniq(wtmp)]
;       index_wl = sort(wtmp)
;       wscale = wtmp[index_wl]

        nbdwc = FLTARR(Ncol/2, Nrow, Npol, Nwave, 2)

        FOR j = 0, Nrow-1 DO BEGIN
            FOR i = 0, Ncol/2-1 DO BEGIN
                FOR k = 0, Npol-1 DO BEGIN
                   nbdwc[i, j, k, *, 0] = REFORM(INTERPOL(REFORM(nbsl[i, j, k, index_wl]), wscale, $
                                                       wscale + REFORM(blsl[i, j, k, index_wl])))
                   nbdwc[i, j, k, *, 1] = REFORM(INTERPOL(REFORM(nbsr[i, j, k, index_wl]), wscale, $
                                                       wscale + REFORM(blsr[i, j, k, index_wl])))
                ENDFOR
            ENDFOR
        ENDFOR

        bshift_dstr = fltarr(Ncol/2, Nrow, Npol, Nwave, 2)
        bshift_dstr[*,*,*,*,0] = blsl
        bshift_dstr[*,*,*,*,1] = blsr

        mwld = mwld2 & mwld2 = 0

        ;;#################################################################
        ;; Mask outer field and fill with average value of inside
        ;;#################################################################

        mask = (aps_mod gt 0.)[0:Ncol/2-1,0:Nrow-1]
        index_out_new = WHERE(mask EQ 0)
        index_in_new = WHERE(mask EQ 1)

        FOR k = 0, Npol-1 DO BEGIN
            FOR i = 0, Nwave-1 DO BEGIN
               tmpl = reform(nbdwc[*,*,k,i,0])
               tmpl[index_out_new] = AVG(tmpl[index_in_new])
               nbdwc[*,*,k,i,0] = tmpl
               tmpr = reform(nbdwc[*,*,k,i,1])
               tmpr[index_out_new] = AVG(tmpr[index_in_new])
               nbdwc[*,*,k,i,1] = tmpr
            ENDFOR
        ENDFOR

        ;;#################################################################
        ;; Show data
        ;;#################################################################

        window, 0, xs = 3*Ncol/2, ys = Nrow
        tvscl, avg(reform(nbdwc[*,*,0,*,0]),2),0,0
        tvscl, avg(reform(nbdwc[*,*,0,*,1]),2),Ncol/2,0
        tvscl, avg(mwld,2),2*Ncol/2,0

        IF (Npol EQ 1) THEN nbdwc = REFORM(nbdwc[*,*,0,*,*])

     ENDIF ;--DUAL

     IF KEYWORD_SET(SINGLE) THEN BEGIN

         print,'---------------------------------------------------'
         print,'Ok, We are in Polarimetric single-beam Mode: sorry!'           
         print,'---------------------------------------------------'

         RETURN,0

;         #################################################################
;          Get speckle reconstruction if existent
;         #################################################################

;         IF KEYWORD_SET(SPECKLE) THEN BEGIN

           
;             Find corresponding speckle reconstruction using
;             data base
           

;            print,'------------------------------------------------------'
;            print,'Identify the speckle reconstruction for the scan      '
;            print,'------------------------------------------------------'

;            bbf = nb_speckle_db.filename
;            FOR i = 0, (SIZE(bbf))[1] -1 DO bbf[i] = (STRMID(bbf[i], $ 
;                                                     (strsplit(bbf[i], '/'))))[n_elements(strsplit(bbf[i], '/'))-1]
;            nbf = (STRMID(s_file, (strsplit(s_file, '/'))))[n_elements(strsplit(s_file, '/'))-1]
;            index = WHERE(bbf EQ nbf)
;            burst = unique_elements(nb_speckle_db.burst[index])
           
;            IF N_ELEMENTS(burst) EQ 1 THEN BEGIN 

;               print,'------------------------------------------------------'
;               print,' One reconstruction found : ', burst
;               print,'------------------------------------------------------'

;               speckle_filename = speckle_path + string(fix(burst), $ 
;                                  FORMAT='("speckle_", I3.3, ".sav")')

;               RESTORE, /verb, speckle_filename

;               simage = TRANSPOSE(simage)
;               simage = reverse(simage)

;               sbbi = SIZE(simage)
;               simage = CONGRID(simage, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
;               simage = ROT(simage, r, CUBIC = -0.5)
;               simage = shift_bicub(simage, sfl[0], sfl[1])

;               simage = simage[0:Ncol-1,0:Nrow-1]

              
;                Shift speckle reconstruction on average whitelight or
;                to correct for atmospheric refraction. This is
;                corrected for in the whitelight already but not in
;                the reconstruction!
              

;               delta = shc((avg(mwld,2))[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], $ 
;                           simage[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], interpolate=1)
;               simage = (shift_bicub(simage,delta[0],delta[1]))[0:Ncol-1,0:Nrow-1]
           
;               simage[index_out] = avg(simage[index_in])

;            ENDIF ;--BURST

;            IF N_ELEMENTS(burst) GT 1 THEN BEGIN

;               print,'------------------------------------------------------'
;               print,' More than one reconstruction found : ', burst
;               print,'------------------------------------------------------'

;               simages = fltarr(Ncol,Nrow, N_ELEMENTS(burst))

;               FOR i = 0, N_ELEMENTS(burst)-1 DO BEGIN

;                   speckle_filename = speckle_path + string(fix(burst[i]), $ 
;                                      FORMAT='("speckle_", I3.3, ".sav")')

;                   RESTORE, /verb, speckle_filename

; ;;                  simage = TRANSPOSE(simage)
;                   simage = reverse(simage)

;                   sbbi = SIZE(simage)
;                   simage = CONGRID(simage, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
;                   simage = ROT(simage, r, CUBIC = -0.5)
;                   simage = shift_bicub(simage, sfl[0], sfl[1])

;                   tmp = simage[0:Ncol-1,0:Nrow-1]

;                   ;;
;                   ;; Shift speckle reconstruction on average whitelight or
;                   ;; to correct for atmospheric refraction. This is
;                   ;; corrected for in the whitelight already but not in
;                   ;; the reconstruction!
;                   ;;

;                   delta = shc(avg(mwld,2), tmp, /interpolate)
;                   tmp = shift_bicub(tmp,delta[0],delta[1])
           
; ;;                  tmp[index_out] = avg(tmp[index_in])

;                   ;;
;                   ;; align references
;                   ;;

;                   IF i EQ 0 THEN ref = tmp
                     
;                   dxy = shc(ref[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], $ 
;                                tmp[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], /interpolate)
;                   tmp = shift_bicub(tmp, dxy[0], dxy[1])

;                   simages[*,*,i] = tmp

;               ENDFOR

;               simage = simages & simages = 0

;            ENDIF ;--BURST

;         ENDIF ;--SPECKLE

;         ;;#################################################################
;         ;; Get telescope geometry and lightlevel if existent
;         ;;#################################################################

;         if n_elements(telparams) gt 0 then begin

;            print,'-----------------------------------------------------'
;            print,'Telescope parameter exist and loaded!                '
;            print,'Light level is extracted from telescope information  '
;            print,'-----------------------------------------------------'

;            tp_azim  = fltarr(Npol*Nwave)
;            tp_elev  = fltarr(Npol*Nwave)
;            tp_table = fltarr(Npol*Nwave)
;            tp_llevel = fltarr(Npol*Nwave)
;            tp_pee = fltarr(Npol*Nwave)
;            tp_bee = fltarr(Npol*Nwave)
;            tp_see = fltarr(Npol*Nwave)
;            tp_slat = fltarr(Npol*Nwave)
;            tp_slng = fltarr(Npol*Nwave)

;            for i=0, Npol*Nwave - 1 do begin
;                tp_azim[i]  = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).azim
;                tp_elev[i]  = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).elev
;                tp_table[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).table
;                tp_llevel[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).llevel
;                tp_pee[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).pee
;                tp_bee[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).bee
;                tp_see[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).see
;                tp_slng[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).slng
;                tp_slat[i] = (find_ibis_tel_params(telparams, (exposures.exposure_start_string)[i], /NB)).slat
;            endfor

;            lightlevel = tp_llevel /  avg(tp_llevel)

;         endif else begin

;               print,'--------------------------------------------------------------------'
;               print,'Light level for polarimetry is extracted from broad band information'
;               print,'--------------------------------------------------------------------'

;               lightlevel = fltarr(Npol*Nwave)
;               for i=0, Npol*Nwave - 1 do lightlevel[i] = avg((mwld[*,*,i])[index_in])
;               lightlevel /= avg(lightlevel)

;         endelse ;--TELPARAMS

;         ;;#################################################################
;         ;; Flatfield and lightlevel correction for nb and wl
;         ;;#################################################################

;         ;; Apply gain table and lightlevel for each wavelength separately.
 
;         print,'--------------------------------------------------------------------'
;         print,'Light level and gain table correction for narrowband                '
;         print,'Light level for whitelight channel                                  '
;         print,'--------------------------------------------------------------------'

;         l = 0
;         mwldn = 0. * mwld
;         nbdwc = fltarr(Ncol, Nrow, dim[3])
;         nbdark = avg(nb_dark[*,*,1:*],2)

;         FOR n=0, Nwave-1 DO BEGIN  
;             FOR i=0, Npol-1 DO BEGIN
;                mwldn[*,*,l] = mwld[*,*,l] / lightlevel[l]
;                nbdwc[*, *, l] = ((nb_data[*, *, l] - nbdark) * wcgain[*, *, index_ff[n], i]) / lightlevel[l]
;                l += 1
;             ENDFOR
;         ENDFOR

;         mwld = mwld
;         mwldn = 0

;         s = size(mwld)

;         ;;#################################################################
;         ;; Do the destretch with IDL code
;         ;;#################################################################

;         print,'-----------------------------------------'
;         print,'Destretch starts now .....               '
;         print,'-----------------------------------------'

;         IF KEYWORD_SET(DSTR_IDL) THEN BEGIN

;            print,'-----------------------------------------'
;            print,'Destretch mode: IDL-code                 '
;            print,'Method : ', method
;            print,'Kernel : ', DS_KERNEL
;            print,'-----------------------------------------'

;            If method eq 'AVG' then begin
;              ref = avg(mwld,2)
;              mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
;            ENDIF

;            IF method EQ 'SEQ' THEN BEGIN
;              mwld2 = destretch_ibis(AVG_IMG[0], mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
;            ENDIF

;            IF method EQ 'SPECKLE' THEN BEGIN

;              ;;
;              ;; ref in this case can be one or >1 image!
;              ;;

;              ref = simage
;              mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)

;            ENDIF

;            ;;#################################################################
;            ;; Destretch narrowband and blueshift
;            ;;#################################################################

;            print,'-------------------------------------------------------'
;            print,'Destretch of whitelight done .....                     '
;            print,'Now we destretch the narrowband and the blueshift map  '
;            print,'-------------------------------------------------------'

;            ;;
;            ;; destretch narrowband
;            ;;

;            nb = 0. * nbdwc
;            for i=0,dim[3]-1 do begin
;                nb[*, *, i] = doreg(nbdwc[*, *, i], grid, shft_tot[*, *, *, i])
;            endfor

;            ;;
;            ;; destretch blueshift
;            ;;

;            bshift_mod = bshift
;            blds = rebin(bshift_mod, Ncol, Nrow, Npol*Nwave)

;            bl = 0. * blds
;            for i=0,dim[3]-1 do begin
;                bl[*, *, i] = doreg(blds[*, *, i], grid, shft_tot[*, *, *, i])
;            endfor

;         ENDIF ;--DSTR_IDL

;         ;;#################################################################
;         ;; Perform blueshift correction
;         ;;#################################################################

;         print,'-------------------------------------------------------'
;         print,'Now we correct the narrowband for the blueshift        '
;         print,'-------------------------------------------------------'

;         nb = REFORM(nb, Ncol, Nrow, Npol, Nwave)
;         bl = REFORM(bl, Ncol, Nrow, Npol, Nwave)

;         wtmp = REFORM(exposures.wavelength)
;         wtmp = wtmp[uniq(wtmp)]
;         index_wl = sort(wtmp)
;         wscale = wtmp[index_wl]

;         nbdwc = FLTARR(Ncol, Nrow, Npol, Nwave)

;         FOR j = 0, Nrow-1 DO BEGIN
;             FOR i = 0, Ncol-1 DO BEGIN
;                 FOR k = 0, Npol-1 DO BEGIN
;                    nbdwc[i, j, k, *] = REFORM(INTERPOL(REFORM(nb[i, j, k, index_wl]), wscale, $
;                                                        wscale + REFORM(bl[i, j, k, index_wl])))
;                 ENDFOR
;             ENDFOR
;         ENDFOR

;         mwld = mwld2 & mwld2 = 0

;         ;;#################################################################
;         ;; Mask outer field and fill with average value of inside
;         ;;#################################################################

;         mask = fix(aps)
;         index_out_new = WHERE(mask EQ 0)
;         index_in_new = WHERE(mask EQ 1)

;         FOR k = 0, Npol-1 DO BEGIN
;             FOR i = 0, Nwave-1 DO BEGIN
;                tmp = reform(nbdwc[*,*,k,i])
;                tmp[index_out_new] = AVG(tmp[index_in_new])
;                nbdwc[*,*,k,i] = tmp
;             ENDFOR
;         ENDFOR

;         ;;#################################################################
;         ;; Show data
;         ;;#################################################################

;         window, 0, xs = 3*Ncol, ys = Nrow
;         tvscl, avg(reform(nbdwc[*,*,0,*]),2),0,0
;         tvscl, avg(reform(nbdwc[*,*,0,*]),2),Ncol,0
;         tvscl, avg(mwld,2),2*Ncol,0

     ENDIF ;--SINGLE

  ENDIF ELSE BEGIN ;--POL

 ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 ;; 
 ;; DONE SPECTROPOLARIMETRIC CASE
 ;;
 ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
 ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        print,'---------------------------------------------------'
        print,'Ok, We are in Spectroscopic Mode                   '
        print,'---------------------------------------------------'

        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; We used to do a lightlevel correction at this point but we
        ;; will not do this anymore until the llevel input extracted
        ;; from the DST guider light level is updated
        ;; with an appropriate cadence. 
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; Flatfield correction for narrowband
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        ;;
        ;; Apply gain table for each wavelength separately.
        ;;
         
        print,'--------------------------------------'
        print,'Gain table correction for narrowband  '
        print,'--------------------------------------'

        mwldn = 0. * bb_data
        nbdwc = 0. * nb_data

        nbdark = avg(dark,2)

        FOR n=0, Nwave-1 DO BEGIN  
            wltmp = bb_data[*,*,n]  ; / lightlevel[n]
            wltmp[index_out] = avg(wltmp[index_in])
            mwldn[*, *, n] = wltmp
            nbtmp = ( (nb_data[*, *, n] - nbdark) * gain[*, *, index_ff[n]] ) ;  / lightlevel[n]
            nbtmp[index_out] = AVG(nbtmp[index_in]) 
            nbdwc[*, *, n] = nbtmp
        ENDFOR

        mwld = mwldn
        mwldn = 0

        s = size(mwld)

        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; Do the destretch with IDL-code
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        print,'-----------------------------------------'
        print,'Destretch starts now .....               '
        print,'-----------------------------------------'

        print,'-----------------------------------------'
        print,'Destretch mode: IDL-code                 '
        print,'Method : ', method
        print,'Kernel : ', DS_KERNEL
        print,'-----------------------------------------'

        dummy = reg(findgen(100,100), findgen(100,100), bytarr(12,12))
        forward_function doreg, mkcps, repair

        ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
        ;; Destretch whitelight
        ;;^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

        IF method EQ 'AVG' THEN BEGIN
           ref = avg(mwld,2)
           mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
        ENDIF

        IF method EQ 'SEQ' THEN BEGIN
           mwld2 = destretch_ibis(AVG_IMG[0], mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)
        ENDIF

        IF method EQ 'EXTERNAL' THEN BEGIN

           restore, ref_file, /verb

           IF (SIZE(image))[0] EQ 2 THEN BEGIN 
 
              tmp = image

              sbbi = SIZE(tmp)

              IF orientation eq 'default' then tmp = reverse(rotate(tmp,1)) 

              tmp = CONGRID(tmp, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
              tmp = rot(tmp,r,cubic=-0.5)
              tmp_n = ibis_set(tmp,sbbi[1],sbbi[2])
              tmp_n = shift_bicub(tmp_n,sfl[0],sfl[1])

              image = tmp_n

           ENDIF

           IF (SIZE(image))[0] EQ 3 THEN BEGIN 

              print,'-------------------------------------------------------------------------'
              print,' More than one reconstruction found for this scan: ', (SIZE(image))[3]
              print,'-------------------------------------------------------------------------'

              images = fltarr(Ncol,Nrow, (SIZE(image))[3])

              FOR i = 0, (SIZE(image))[3]-1 DO BEGIN

                  tmp = reform(image[*,*,i])

                  sbbi = SIZE(tmp)

                  IF orientation eq 'default' then tmp = reverse(rotate(tmp,1)) 

                  tmp = CONGRID(tmp, f1 * sbbi[1], f2 * sbbi[2], CUBIC = -0.5)
                  tmp = rot(tmp,r,cubic=-0.5)
                  tmp_n = ibis_set(tmp,sbbi[1],sbbi[2])
                  tmp_n = shift_bicub(tmp_n,sfl[0],sfl[1])
                
                  ;;
                  ;; references should be aligned already
                  ;;

                  ;;IF i EQ 0 THEN ref = tmp_n
                     
                  ;;   dxy = shc(ref[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], $ 
                  ;;                 tmp_n[Ncol/2-128:Ncol/2+128,Nrow/2-128:Nrow/2+128], /interpolate)
                  ;;   tmp = shift_bicub(tmp_n, dxy[0], dxy[1])

                  images[*,*,i] = tmp_n

              ENDFOR

              image = images

           ENDIF

           ;;
           ;; ref in this case can be one or >1 image!
           ;;

           ref = image
           mwld2 = destretch_ibis(ref, mwld, DS_KERNEL, REGION, DT_DETR, shifts=shft_tot, grid=grid)

        ENDIF

        mwld = mwld2

        ;;#################################################################
        ;; Destretch narrowband and blueshift array
        ;;#################################################################

        print,'-------------------------------------------------------'
        print,'Destretch of whitelight done .....                     '
        print,'Now we destretch the narrowband and the blueshift map  '
        print,'-------------------------------------------------------'

        IF N_ELEMENTS(DS_KERNEL) EQ 1 THEN BEGIN
              
           print,'Align shift :', shft_tot[*, 0, 0, 0]

           nb = 0. * nbdwc
           FOR i = 0,dim[3]-1 DO BEGIN
               nb[*, *, i] = shift_bicub(nbdwc[*, *, i], shft_tot[0, 0, 0, i], shft_tot[1, 0, 0, i])
           ENDFOR

           blds = rebin(bshift, Ncol, Nrow, Npol*Nwave)
           bshift_dtsr = 0. * blds
           FOR i = 0,dim[3]-1 DO BEGIN
               bshift_dstr[*, *, i] = shift_bicub(blds[*, *, i], shft_tot[0, 0, 0, i], shft_tot[1, 0, 0, i])
           ENDFOR

        ENDIF ELSE BEGIN

              ;;
              ;; destretch narrowband
              ;;

              nb = 0. * nbdwc
              for i=0,s[3]-1 do begin
                  nb[*, *, i] = doreg(nbdwc[*, *, i], grid, shft_tot[*, *, *, i])
              endfor

              ;;
              ;; destretch blueshift
              ;;

              blds = rebin(bshift, Ncol, Nrow, Npol*Nwave)

              bshift_dstr = 0. * blds
              for i=0,s[3]-1 do begin
                  bshift_dstr[*, *, i] = doreg(blds[*, *, i], grid, shft_tot[*, *, *, i])
              endfor

        ENDELSE

        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; Perform blueshift correction
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        nbdwc = FLTARR(s[1],s[2],Nwave)

        FOR i = 0, s[1]-1 DO BEGIN
            FOR j = 0, s[2]-1 DO BEGIN

                nbdwc[i, j, *] = REFORM(INTERPOL(REFORM(nb[i, j, index_wl]), wscale, $
                                                 wscale + REFORM(bshift_dstr[i, j, *])))
            ENDFOR
        ENDFOR

        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; Mask outer field and fill with average value of inside
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

        maperture = ibis_aperture(reform(mask),/plot)
        mask_new = shift(dist(Ncol), Ncol/2, Nrow/2)
        mask_new = mask_new LT (maperture.radius-5)
        index_out_new = WHERE(mask_new EQ 0)
        index_in_new = WHERE(mask_new EQ 1)

;        IF (SIZE(image))[0] EQ 2 THEN image[index_out] = AVG(image[index_in_new])
                
;        IF (SIZE(image))[0] EQ 3 THEN BEGIN

;           for i = 0, ((SIZE(image))[3])-1 DO BEGIN
               
;               tmp = REFORM(image[*,*,i])
;               tmp[index_out] = AVG(tmp[index_in_new])
;               image[*,*,i] = tmp

;           ENDFOR

;        ENDIF

        FOR i = 0, s[3]-1 DO BEGIN

            tmp = reform(mwld[*,*,i])
            tmp[index_out_new] = AVG(tmp[index_in_new])
            mwld[*,*,i] = tmp
            tmp = reform(nbdwc[*,*,i])
            tmp[index_out_new] = AVG(tmp[index_in_new])
            nbdwc[*,*,i] = tmp

        ENDFOR

        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
        ;; Save data outside
        ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

       ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
       ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
       ;;
       ;; DONE SPECTROSCOPIC CASE
       ;;
       ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
       ;;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


    ENDELSE ;; POLARIMETRIC OR SPECTROSCOPIC CASE

;;***************************************************************
;; Done
;;***************************************************************

RETURN, nbdwc

END
