;;*****************************************************************************************
;;*****************************************************************************************
;; MAIN IBIS EXECUTION SCRIPT
;;*****************************************************************************************
;;*****************************************************************************************
PRO ibis_exe

@ibis_config.dat

;;*****************************************************************************************
;;*****************************************************************************************
;; GENERATE DATABASE
;;*****************************************************************************************
;;*****************************************************************************************

;IF FLAG_DATABASE THEN BEGIN
;
;   ibis_database, spath, wpath, date, SAVEFILE = database_file
;
;ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; GENERATE AVERAGED FLATS AND DARKS FOR NARROWBAND AND BROADBAND
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_FLAT_DARK_BB THEN BEGIN

   ibis_flat_dark_avg_bb, ffdir_bb, dcdir_bb, ffile_bb, dfile_bb

ENDIF

IF FLAG_FLAT_DARK_NB THEN BEGIN

   ibis_flat_dark_avg_nb, ffdir_nb, dcdir_nb, ffile_nb, dfile_nb, $ 
                          nstart_ext, nend_ext

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; DETERMINE ALIGNMENT PARAMETERS BETWEEN NARROWBAND AND BROADBAND
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_ALIGN THEN BEGIN

   ibis_align, griddir_nb, griddir_bb, $ 
               nstart_ext, nend_ext, $ 
               ORIENTATION = orientation, METHOD = method_align, $ 
               SAVEFILE1 = afile, SAVEFILE2 = gsfile, SAVEFILE3 = gwfile

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; COMBINE NARROWBAND AND BROADBAND
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_PREPARE THEN BEGIN

   ibis_prepare, datadir_nb, datadir_bb, dir_tmp_nb, dir_tmp_bb, $
                 dfile_bb, ffile_bb, $ 
                 nstart_ext, nend_ext, $
                 afile, filter, $;filter_pos, $ 
                 ORIENTATION = orientation,$
                 NREPEAT = nrepeat
     

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; CALCULATE SYSTEMATIC WAVELENGTHSHIFT
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_WAVELENGTH_SHIFT THEN BEGIN

   offset = ibis_wavelength_shift(ffile_nb, dfile_nb, $
                                  npoints, aps, $
                                  SAVEFILE = bfile, $ 
                                  POL = pol, DUAL = dual, SINGLE = single, STOKES = stokes)

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; GENERATE NARROWBAND GAIN
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_GAIN THEN BEGIN

   ibis_gain_nb, ffile_nb, dfile_nb, pfile, bfile, gfile, $ 
                 WSHIFT = wshift, $ 
                 POL = pol, DUAL = dual, SINGLE = single, STOKES = stokes

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; IMAGE RECONSTRUCTION PART -- OPTIONAL
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_SPECKLE THEN BEGIN



    ibis_speckle, ref_indir, ref_outdir, dfile_bb, ffile_bb, nstart_ext, nend_ext, $ 
                 NREPEAT = nrepeat, SWAP = swap

ENDIF

IF FLAG_EXTERNAL THEN BEGIN

   ibis_external, ref_outdir, ref_outdir,  BIN = bin, SWAP = swap

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; MAIN SPECTROSOCPIC CALIBRATION
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_CALIB THEN BEGIN

   ibis_main, dir_tmp_nb, dir_tmp_bb, rdir_nb, rdir_bb, $ 
              gfile, dfile_nb, afile, bfile, filter,$ 
              WSHIFT = wshift, DSTR_IDL = dstr_idl, SCAN = scan, ORIENTATION = orientation, $
              POL = pol, SINGLE = single, DUAL = dual, STOKES = stokes

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;;POLARIZATION CALIBRATION CURVES
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_POLCALCURVE THEN BEGIN

   ibis_polcalcurve, pol_indir, pol_outdir, filter_pos, $
                     SAVEFILE1 = pfile1, SAVEFILE2 = pfile2, $
                     SINGLE = single, DUAL = dual
 
ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;;POLARIMETER RESPONSE: X-MATRICES
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_XMATRICES THEN BEGIN

   ibis_xmatrix, polcalcurve_infile, xmat_outdir, filter, date, $
                 TMAT = tmat

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; MAIN SPECTROPOLARIMETRIC CALIBRATION
;;*****************************************************************************************
;;*****************************************************************************************

IF FLAG_CALIB_POL THEN BEGIN

   ibis_main_polarization, i_rdir_nb, o_rdir_nb, filter, $
                           tfile, xmat1, xmat2, $
                           SCAN = scan, MASK = mask, ROI = roi, WRANGE = wrange, $
                           SINGLE = single, DUAL = dual, STOKES = stokes, $
                           RESIDUAL = residual, EXTERNALC = externalc, EXTERNAL_RESIDUAL_FILE = external_residual_file 

ENDIF

;;*****************************************************************************************
;;*****************************************************************************************
;; DONE
;;*****************************************************************************************
;;*****************************************************************************************

END
