;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_gain_nb
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_gain_nb, f_file, d_file, p_file, b_file, savefile, WSHIFT = wshift, $ 
               Pol = pol, SMOOTH = smooth, SINGLE = single, DUAL = dual, $
               STOKES = stokes
               
  ;; restore mean dark, mean flat and prefilter file

  restore, f_file, /verb
  restore, d_file, /verb
  restore, p_file, /verb
  restore, b_file, /verb

  IF KEYWORD_SET(WSHIFT) eq 0 then bshift = offset.poly_fit

  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'poly_fit' then bshift = offset.poly_fit
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'poly'     then bshift = offset.poly
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'cog'      then bshift = offset.cog
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'cog_fit'  then bshift = offset.cog_fit

  ;; get wavelength scale and make equidistant on 10 mA grid


  wscale_equi = ibis_equidistant(info_flat_nb, wscale)
  nwscale = n_elements(wscale_equi)

;  wscale      = REFORM((unique_elements(info_flat_nb.grid)))
;  wscale      = wscale[sort(wscale)]
;  nwscale     = ROUND((MAX(wscale) - MIN(wscale))/0.01d)          
;  wscale_equi = FINDGEN(nwscale)/(nwscale-1) $ 
;                * (MAX(wscale) - MIN(wscale)) + MIN(wscale)

  ;; Get dimensions

  dim   = SIZE(flat)
  Nx    = dim[1]
  Ny    = dim[2]
  IF KEYWORD_SET(STOKES) THEN Npol = stokes ELSE Npol = 1
  Nwave = dim[3]/Npol

  ;; Order flats when in rabbit mode
     
  wtmp = REFORM((unique_elements(info_flat_nb.grid)))
  index = sort(wtmp)

  ftmp = flat

  FOR i = 0, Nwave-1 DO ftmp[*,*,i*Npol:(i+1)*Npol-1] = flat[*,*,index[i]*Npol:(index[i]+1)*Npol-1]
 
  flat = ftmp & ftmp = 0

  ;; create average dark

  dark = avg(dark, 2)
  
  ;; interior of the aperture
     
  temp = fltarr(Nx, Ny)

;; Get the outside indeces for whole field

  index_out = where(aps eq 0)
  index_in = where(aps ge 1)

;;---------------------------------------------------------
;;---------------------------------------------------------
;; Distinguish Modes : Spectroplarimetric modes
;;---------------------------------------------------------
;;---------------------------------------------------------

  IF keyword_set(POL) then begin

     print,'---------------------------------------------'
     print,'Ok, we are in spectropolarimetric mode ....  '
     print,'---------------------------------------------'

     IF keyword_set(DUAL) then begin

        print,'---------------------------------------------'
        print,'DUAL-BEAM mode ....  '
        print,'---------------------------------------------'

        ;; Get the full aperture and for left and right channel

        idx  = where(aps ge 1, Num)
        idxx = idx mod Nx
        idxy = idx / Nx

        idxl  = where_n(aps[0:Nx/2-1,*] ge 1)
        idxlx = idxl[*,0]
        idxly = idxl[*,1]
        Numl = n_elements(idxlx)

        idxr  = where_n(aps[Nx/2:Nx-1,*] ge 1)
        idxrx = idxr[*,0]
        idxry = idxr[*,1]
        Numr = n_elements(idxrx)

        ;; divide offset into left and right

        offl = bshift[0:Nx/2-1,*]
        offr = bshift[Nx/2:Nx-1,*]

        ;; dark correction

        Nwave = dim[3]/Npol
        wcgain = fltarr(Nx, Ny, Nwave, Npol)

        FOR n=0, Nwave-1 DO begin
            for i=0, Npol-1 do begin
                temp[index_in] = (reform(flat[*,*,n*Npol+i]) - dark)[index_in]
                temp[index_out] = 1.
                wcgain[*, *, n, i] = temp
            ENDFOR
        endfor

        wcgain_l = wcgain[0:Nx/2-1,*,*,*]
        wcgain_r = wcgain[Nx/2:Nx-1,*,*,*]

        ;; Calculate "super-bad profile" on fine grid for both channels separately
        ;; Interpolate in wavelength to shift the wavelength scale at
        ;; each valid pixel position back to a reference

        sbpprof_l = fltarr(nwscale, Npol)
        sbpprof_r = fltarr(nwscale, Npol)

        tmpl = fltarr(Nx/2, Ny, nwscale, Npol)
        tmpr = fltarr(Nx/2, Ny, nwscale, Npol)

        for j=0, Npol-1 do begin

            FOR i=0L, Numl-1 DO BEGIN
                p = interpol(reform(wcgain_l[idxlx[i], idxly[i], *,j]), wscale, wscale_equi+offl[idxlx[i], idxly[i]], /quadratic)
                tmpl[idxlx[i], idxly[i], *,j] = p
                sbpprof_l[*,j] += p
            endfor

            sbpprof_l[*,j] /= FLOAT(Numl)

            FOR i=0L, Numr-1 DO BEGIN
                p = interpol(reform(wcgain_r[idxrx[i], idxry[i], *,j]), wscale, wscale_equi+offr[idxrx[i], idxry[i]], /quadratic)
                tmpr[idxrx[i], idxry[i], *,j] = p
                sbpprof_r[*,j] += p
            endfor

            sbpprof_r[*,j] /= FLOAT(Numr)

        ENDFOR

        ;; Create shifted array to "gain" the mean flat array.

        wcgain_tmpl = 0. * wcgain[0:Nx/2-1,*,*,*]
        wcgain_tmpr = 0. * wcgain[Nx/2:Nx-1,*,*,*]
        wcgain_shift = 0. * wcgain

        for j=0, Npol-1 do begin
            for i=0L, Numl-1 do begin

                wscale_blue  = wscale - offl[idxlx[i], idxly[i]]
                wcgain_tmpl[idxlx[i], idxly[i], *,j] = INTERPOL(reform(sbpprof_l[*,j]), wscale_equi, wscale_blue, /quadratic)

            endfor

            for i=0L, Numr-1 do begin

                wscale_blue  = wscale - offr[idxrx[i], idxry[i]]
                wcgain_tmpr[idxrx[i], idxry[i], *,j] = INTERPOL(reform(sbpprof_r[*,j]), wscale_equi, wscale_blue, /quadratic)

            endfor

        endfor

        wcgain_shift[0:Nx/2-1,*,*,*] = wcgain_tmpl
        wcgain_shift[Nx/2:Nx-1,*,*,*] = wcgain_tmpr

        ;; Calculate gain (1 / (devide flat by shifted sbpprof))

        wcgain_final = wcgain_shift / wcgain

     ENDIF

     IF keyword_set(SINGLE) then begin

        print,'---------------------------------------------'
        print,'SINGLE-BEAM mode ....  '
        print,'---------------------------------------------'

        idx  = where_n(aps GE 1)
        idxx = idx[*,0]
        idxy = idx[*,1]
        Num = N_ELEMENTS(idxx)

        off = bshift

        ;; dark correction

        Nwave = dim[3]/Npol
        wcgain = fltarr(Nx, Ny, Nwave, Npol)

        FOR n=0, Nwave-1 DO begin
            for i=0, Npol-1 do begin
                temp[index_in] = (reform(flat[*,*,n*Npol+i]) - dark)[index_in]
                temp[index_out] = 1.
                wcgain[*, *, n, i] = temp
            ENDFOR
        endfor

        wcgain = wcgain

        ;; Calculate "super-bad profile" on fine grid.
        ;; Interpolate in wavelength to shift the wavelength scale at
        ;; each valid pixel position back to a reference

        sbpprof = fltarr(nwscale, Npol)

        tmp = fltarr(Nx, Ny, nwscale, Npol)

        for j=0, Npol-1 do begin

            FOR i=0L, Num-1 DO BEGIN
                p = interpol(reform(wcgain[idxx[i], idxy[i], *,j]), wscale, wscale_equi+off[idxx[i], idxy[i]], /quadratic)
                tmp[idxx[i], idxy[i], *,j] = p
                sbpprof[*,j] += p
            endfor

            sbpprof[*,j] /= FLOAT(Num)

        ENDFOR

        ;; Create shifted array to "gain" the mean flat array.

        wcgain_tmp = 0. * wcgain
        wcgain_shift = 0. * wcgain

        for j=0, Npol-1 do begin
            for i=0L, Num-1 do begin

                wscale_blue  = wscale - off[idxx[i], idxy[i]]
                wcgain_tmp[idxx[i], idxy[i], *,j] = INTERPOL(reform(sbpprof[*,j]), wscale_equi, wscale_blue, /quadratic)

            endfor
        endfor

        wcgain_shift = wcgain_tmp

        ;; Calculate gain (1 / (devide flat by shifted sbpprof))

        wcgain_final = wcgain_shift / wcgain

     ENDIF

     If keyword_set(SMOOTH) then begin

        ;; and now the voodoo starts.
        ;; the voodoo is ment to remove residual solar variations present
        ;; in the gain. 

        print,'---------------------------------------------'
        print,'Ok, we smooth ....                           '
        print,'---------------------------------------------'

        smooth_kernel = smooth

        wcgain_smooth = 0. * wcgain_final

        wscale_even = findgen(nwscale)/(nwscale-1) $ 
                               * (max(wscale) - min(wscale)) + min(wscale)
        
        for j=0, Npol-1 do begin
            for i=0L, Num-1 do begin
            
                tmp = reform(wcgain_final[idxx[i], idxy[i], *, j])
                tmp_new = INTERPOL(tmp, wscale, wscale_even, /quadratic)
                tmp_sm = smooth(tmp_new, smooth_kernel, /edge_trunc)
                tmp = INTERPOL(tmp_sm, wscale_even, wscale, /quadratic)
                wcgain_smooth[idxx[i], idxy[i], *, j] = tmp

            endfor
        endfor

        wcgain_final = wcgain_smooth

     endif

     ;; now correct outside FOV

     FOR n=0, Nwave-1 DO begin
         for i = 0,Npol-1 do begin

             tmp = reform(wcgain_final[*,*,n,i])
             tmp[index_out] = 1.
             wcgain_final[*,*,n,i] = tmp

         endfor
     endfor   

  endif else begin

;;---------------------------------------------------------
;;---------------------------------------------------------
;; Distinguish Modes : Spectroscopic mode
;;---------------------------------------------------------
;;---------------------------------------------------------

     print,'---------------------------------------------'
     print,'Ok, we are in spectroscopic mode ....  '
     print,'---------------------------------------------'

     ;; Get the aperture for this field

     idx  = where(aps ge 1, Num)
     idxx = idx mod Nx
     idxy = idx / Nx

     Nwave = dim[3]
     wcgain = fltarr(Nx, Ny, Nwave)

     FOR n=0, Nwave-1 DO begin
         temp[idx] = (reform(flat[*,*,n]) - dark)[idx]
         temp[index_out] = 1.
         wcgain[*, *, n] = temp
     endfor

     ;; Calculate "super-bad profile" on fine grid!!!
     ;; Interpolate in wavelength to shift the wavelength scale at
     ;; each valid pixel position back to a reference

     sbpprof = fltarr(nwscale)
     tmp = fltarr(Nx,Ny,nwscale)

     FOR i=0L, Num-1 DO BEGIN

         p = interpol(reform(wcgain[idxx[i], idxy[i], *]), wscale, wscale_equi+bshift[idxx[i], idxy[i]], /quadratic)
         tmp[idxx[i], idxy[i], *] = p
         sbpprof[*] += p

     ENDFOR

     sbpprof /= float(Num)

     ;; Create shifted array to "gain" the mean flat array.

     wcgain_shift = 0. * wcgain

     for i=0L, Num-1 do begin

         wscale_blue  = wscale - bshift[idxx[i], idxy[i]]
         wcgain_shift[idxx[i], idxy[i], *] = INTERPOL(sbpprof, wscale_equi, wscale_blue, /quadratic)

     endfor

     ;; Calculate gain (1 / (devide flat by shifted sbpprof))

     wcgain_final = wcgain_shift / wcgain

     If keyword_set(SMOOTH) then begin

        smooth_kernel = smooth

        ;; and now the voodoo starts.
        ;; the voodoo is ment to remove residual solar variations present
        ;; in the gain. 

        wcgain_smooth = 0. * wcgain_final

        wscale_even = findgen(n_elements(wscale))/(n_elements(wscale)-1) $ 
                               * (max(wscale) - min(wscale)) + min(wscale)

        ;; smoothing process

        for i=0L, Num-1 do begin

            tmp = reform(wcgain_final[idxx[i], idxy[i], *])
            tmp_new = INTERPOL(tmp, wscale, wscale_even, /quadratic)
            tmp_sm = smooth(tmp_new, smooth_kernel, /edge)
            tmp = INTERPOL(tmp_sm, wscale_even, wscale, /quadratic)
            wcgain_smooth[idxx[i], idxy[i], *] = tmp

        endfor

        wcgain_final = wcgain_smooth

    endif

    ;; now correct outside FOV

    for i=0,dim[3]-1 do begin

        tmp = reform(wcgain_final[*,*,i])
        tmp[index_out] = 1.
        wcgain_final[*,*,i] = tmp

    endfor   

  endelse

  ;; Store the gain tables for each wavelength 

  wcgain = wcgain_final

      print,'-------------------------------------'
      print,'IBIS gain saved : ', savefile
      print,'-------------------------------------'

  save, FILENAME = savefile, wcgain

END
