;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_main
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_main, i_dir_nb, i_dir_bb, o_dir_nb, o_dir_bb, $
               g_file, d_file, a_file, b_file, filter, $ 
               WSHIFT = wshift, DSTR_IDL = dstr_idl, SCAN = scan, ORIENTATION = orientation, $
               POL = pol, SINGLE = single, DUAL = dual, STOKES = stokes

;;***************************************************************
;; Data files
;;***************************************************************

     sfiles = file_search( i_dir_nb + filter + '_nb*.sav', COUNT = nnb)
     wfiles = file_search( i_dir_bb + filter + '_bb*.sav', COUNT = nbb)

     PRINT,'---------------------------------------'
     PRINT,'Narrowband files found: ', nnb
     PRINT,'Broadband files found: ', nbb
     PRINT,'---------------------------------------'

;;***************************************************************
;; Speckle references if applicable
;;***************************************************************

  If keyword_set(DSTR_IDL) then begin

     If (dstr_idl.method) EQ 'EXTERNAL' then external_filename = (dstr_idl.external_filename)

  endif

;;***************************************************************
;; Restore aux data
;;***************************************************************

  ;;
  ;; Restore files.
  ;; 

  restore, /verb, g_file
  restore, /verb, d_file
  restore, /verb, a_file
  restore, /verb, b_file

  IF KEYWORD_SET(WSHIFT) eq 0 then bshift = offset.poly_fit
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'poly_fit' then bshift = offset.poly_fit
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'poly' then bshift = offset.poly
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'cog' then bshift = offset.cog
  IF KEYWORD_SET(WSHIFT) eq 1 AND wshift eq 'cog_fit' then bshift = offset.cog_fit

  IF KEYWORD_SET(STOKES) THEN Npol = stokes ELSE Npol = 1

;;***************************************************************
;; Start and end of calibration
;;***************************************************************

  If keyword_set(SCAN) then begin

     nstart = scan[0]
     nend   = scan[1]

  ENDIF else begin

        nstart = 0
        nend = nnb-1

  endelse

;;***************************************************************
;; Calibrate 
;;***************************************************************

  for i = nstart, nend do begin

      restore, sfiles[i], /verb
      restore, wfiles[i], /verb

      file1 = (STRMID(sfiles[i], (STRSPLIT(sfiles[i], '/'))))[N_ELEMENTS(STRSPLIT(sfiles[i], '/'))-1]
      file2 = (STRMID(wfiles[i], (STRSPLIT(wfiles[i], '/'))))[N_ELEMENTS(STRSPLIT(wfiles[i], '/'))-1]
      file3 = strmid(file1,0,5) + 'aux' + strmid(file1,7)

      outfile1 = o_dir_nb + file1
      outfile2 = o_dir_bb + file2
      outfile3 = o_dir_bb + file3

      If (dstr_idl.method) EQ 'EXTERNAL' then begin 

         ref_file = dstr_idl.external_filename[i]

         nb_data_dstr = ibis_calib(nb_data, bb_data, wcgain, dark, info_nb, info_bb, dstr_idl, parameter, $ 
                                   bshift, aps, Npol, bb_data_dstr, shifts, bshift_dstr, $ 
                                   POL = pol, DUAL = dual, SINGLE = single, $ 
                                   REF_FILE = ref_file, ORIENTATION = orientation)

         save, filename = outfile1, nb_data_dstr, info_nb, /verb
         save, filename = outfile2, bb_data_dstr, info_bb, /verb
         save, filename = outfile3, shifts, aps, bshift_dstr, dstr_idl, /verb

         print,'*******************************'
         print,'saved: ', outfile1
         print,'saved: ', outfile2
         print,'saved: ', outfile3
         print,'*******************************'

         mwld = 0 & shifts = 0 & bb_data_dstr = 0 & bshift_dstr = 0


      ENDIF


      If (dstr_idl.method) NE 'EXTERNAL' then BEGIN 

         nb_data_dstr = ibis_calib(nb_data, bb_data, wcgain, dark, info_nb, info_bb, dstr_idl, parameter, $
                                   bshift, aps, Npol, bb_data_dstr, shifts, bshift_dstr, $
                                   POL = pol, DUAL = dual, SINGLE = single, $ 
                                   ORIENTATION = orientation)

         save, filename = outfile1, nb_data_dstr, info_nb, /verb
         save, filename = outfile2, bb_data_dstr, info_bb, /verb
         save, filename = outfile3, shifts, aps, bshift_dstr, dstr_idl, /verb

         print,'*******************************'
         print,'saved: ', outfile1
         print,'saved: ', outfile2
         print,'saved: ', outfile3
         print,'*******************************'

         mwld = 0 & shifts = 0 & bb_data_dstr = 0 & bshift_dstr = 0

      ENDIF

  endfor

;;***************************************************************
;; Done
;;***************************************************************

END
