;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;+
; NAME:
;       ibis_speckle
;
; PURPOSE:
;
; CATEGORY:
;
; CALLING SEQUENCE:
;       
;
; INPUTS:
;
; KEYWORD PARAMETERS:
;
; OUTPUTS:
;
;
; OPTIONAL OUTPUTS:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
;           2011-Nov, Alexandra Tritschler, NSO
;-
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
PRO ibis_speckle, input_dir, output_dir, wldfile, wlffile, nstart, nend, $ 
                  NREPEAT = nrepeat, SWAP = swap

;;***************************************************************
;; Data
;;***************************************************************

     wfiles = file_search( input_dir + '*.ScienceObservation.fits', COUNT = nbb)
     ni = nend - nstart + 1 
     sz = read_size(wfiles[0])

     PRINT,'---------------------------------------'
     PRINT,'Science broadband files found: ', nbb
     PRINT,'---------------------------------------'

;;***************************************************************
;; Output
;;***************************************************************

     data_out = 'ibis_bb.'
     ofn = output_dir + data_out

;;***************************************************************
;; Load aux files
;;***************************************************************

     RESTORE, /VERB, wldfile
     RESTORE, /VERB, wlffile

;;***************************************************************
;; Create mask for gain
;;***************************************************************

     msk = create_aperture(flat, 256, 64, 0.65, 11)
     mask = msk
     brush = [[0,1,0],[1,1,1],[0,1,0]]
     FOR i = 0,10 DO mask = ERODE(mask, brush)   
     index_msk = WHERE(mask EQ 1, COMPLEMENT = index0)
     gain  = flat / AVG( flat[index_msk] )

;;***************************************************************
;; Loop through data
;;***************************************************************

     l = 0

     FOR n = 0, nbb-1 DO BEGIN

         print,'Broadband file : ', wfiles[n]

         FOR k = 0, nrepeat - 1 DO BEGIN

             tofn = ofn + STRTRIM(STRING(l,format='(I3.3)'),2)

             OPENW, lun, tofn, /get_lun

             FOR i = 1L, ni DO BEGIN

                 tmp = (READFITS(wfiles[n], EXT = nstart+(k*ni)+i-1, header, /SILENT) - dark ) / gain

                 tmp[index0] = AVG(tmp[index_msk])

                 TVSCL, REBIN(tmp, sz[0]/2, sz[1]/2)
 
                 IF keyword_set(SWAP) then BYTEORDER, tmp, /LSWAP

                 WRITEU, lun, tmp

             ENDFOR

             l = l + 1
            
             CLOSE, lun & free_lun, lun

             PRINT,'saved: ', tofn

         ENDFOR

     ENDFOR

;;***************************************************************
;; Done
;;***************************************************************

END

