pro find_ps

; DST plate scale in [arcs/mm]
dstplatescale = 3.76
; grid line distance [mm]
gridlinedist = 1.0

; line thickness of grid in pixels
lthick = 21

restore,'grid_wl.sav'
img = rot((float(bbtar))[200:600,200:600],35.,cub=-0.5)

; make binary array
img -= mean(img)
img *= -1
img[where(img le 0)] = 0
img[where(img gt 0)] = 1

; hough transform
htf  = hough(img, rho=rho, theta=theta)
dhtf = size(htf, /dimen)

; find grid lines in Hough transform
mp = fltarr(dhtf[0])
for i=0,dhtf[0]-1 do mp[i] = max(htf[i,*])

; find max row, Step 1
idx1 = where(mp eq max(mp))
idx2 = [(idx1[0]-lthick/2+dhtf[0]) mod dhtf[0], (idx1[0]+lthick/2+dhtf[0]) mod dhtf[0]]
if (idx2[0] gt idx2[1]) then begin
  t1 = reform(rebin([htf[0:idx2[1], *], htf[idx2[0]:dhtf[0]-1, *]], 1, dhtf[1]))
  ; zero out the locations
  mp[0:idx2[1]]=0 & mp[idx2[0]:dhtf[0]-1]=0
endif else begin
  t1 = reform(rebin(htf[idx2[0]:idx2[1], *], 1, dhtf[1]))
  ; zero out the locations
  mp[idx2[0]:idx2[1]]=0
endelse
; find auto-correlation and smooth
act1 = smooth(float(fft(fft(t1-mean(t1))*conj(fft(t1-mean(t1))),+1)),3,/edge)
; normalize
act1/= act1[0]
; derivative
dact1 = act1[0:dhtf[1]/2] - shift(act1[0:dhtf[1]/2],1)
; this should ignore the first local max
for i=lthick/2, dhtf[1]/2 do begin
  if ( (dact1[i-1] > 0) AND (dact1[i+1] < 0) ) then break
endfor
max1 = i
; interpolate
denom = act1[max1]*2 - act1[max1-1] - act1[max1+1]
max1 = (max1-.5) + (act1[max1]-act1[max1-1])/denom

; find max row, Step 2
idx1 = where(mp eq max(mp))
idx2 = [(idx1[0]-lthick/2+dhtf[0]) mod dhtf[0], (idx1[0]+lthick/2+dhtf[0]) mod dhtf[0]]
if (idx2[0] gt idx2[1]) then begin
  t2 = reform(rebin([htf[0:idx2[1], *], htf[idx2[0]:dhtf[0]-1, *]], 1, dhtf[1]))
  ; zero out the locations
  mp[0:idx2[1]]=0 & mp[idx2[0]:dhtf[0]-1]=0
endif else begin
  t2 = reform(rebin(htf[idx2[0]:idx2[1], *], 1, dhtf[1]))
  ; zero out the locations
  mp[idx2[0]:idx2[1]]=0
endelse
; find auto-correlation and smooth
act2 = smooth(float(fft(fft(t2-mean(t2))*conj(fft(t2-mean(t2))),+1)),3,/edge)
; normalize
act2/= act2[0]
; derivative
dact2 = act2[0:dhtf[1]/2] - shift(act2[0:dhtf[1]/2],1)
; this should ignore the first local max
for i=lthick/2, dhtf[1]/2 do begin
  if ( (dact2[i-1] > 0) AND (dact2[i+1] < 0) ) then break
endfor
max2 = i
; interpolate
denom = act2[max2]*2 - act2[max2-1] - act2[max2+1]
max2  = (max2-.5) + (act2[max2]-act2[max2-1])/denom

; compute image scale
mmpx_x = gridlinedist/max1
mmpx_y = gridlinedist/max2

asppx_x = dstplatescale * mmpx_x
asppx_y = dstplatescale * mmpx_y

stop

end
